#!/usr/bin/env python
# Copyright (c) 2012 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""This script is used to download prebuilt clang binaries."""

import os
import shutil
import subprocess
import stat
import sys
import tarfile
import tempfile
import time
import urllib2


# CLANG_REVISION and CLANG_SUB_REVISION determine the build of clang
# to use. These should be synced with tools/clang/scripts/update.py in
# Chromium.
CLANG_REVISION = '328575'
CLANG_SUB_REVISION=1

PACKAGE_VERSION = "%s-%s" % (CLANG_REVISION, CLANG_SUB_REVISION)

# Path constants. (All of these should be absolute paths.)
THIS_DIR = os.path.abspath(os.path.dirname(__file__))
LLVM_BUILD_DIR = os.path.join(THIS_DIR, 'llvm-build')
STAMP_FILE = os.path.join(LLVM_BUILD_DIR, 'cr_build_revision')

# URL for pre-built binaries.
CDS_URL = os.environ.get('CDS_CLANG_BUCKET_OVERRIDE',
    'https://commondatastorage.googleapis.com/chromium-browser-clang')

# Bump after VC updates.
DIA_DLL = {
  '2013': 'msdia120.dll',
  '2015': 'msdia140.dll',
  '2017': 'msdia140.dll',
}


def DownloadUrl(url, output_file):
  """Download url into output_file."""
  CHUNK_SIZE = 4096
  TOTAL_DOTS = 10
  num_retries = 3
  retry_wait_s = 5  # Doubled at each retry.

  while True:
    try:
      sys.stdout.write('Downloading %s ' % url)
      sys.stdout.flush()
      response = urllib2.urlopen(url)
      total_size = int(response.info().getheader('Content-Length').strip())
      bytes_done = 0
      dots_printed = 0
      while True:
        chunk = response.read(CHUNK_SIZE)
        if not chunk:
          break
        output_file.write(chunk)
        bytes_done += len(chunk)
        num_dots = TOTAL_DOTS * bytes_done / total_size
        sys.stdout.write('.' * (num_dots - dots_printed))
        sys.stdout.flush()
        dots_printed = num_dots
      if bytes_done != total_size:
        raise urllib2.URLError("only got %d of %d bytes" %
                               (bytes_done, total_size))
      print ' Done.'
      return
    except urllib2.URLError as e:
      sys.stdout.write('\n')
      print e
      if num_retries == 0 or isinstance(e, urllib2.HTTPError) and e.code == 404:
        raise e
      num_retries -= 1
      print 'Retrying in %d s ...' % retry_wait_s
      time.sleep(retry_wait_s)
      retry_wait_s *= 2


def EnsureDirExists(path):
  if not os.path.exists(path):
    print "Creating directory %s" % path
    os.makedirs(path)


def DownloadAndUnpack(url, output_dir):
  with tempfile.TemporaryFile() as f:
    DownloadUrl(url, f)
    f.seek(0)
    EnsureDirExists(output_dir)
    tarfile.open(mode='r:gz', fileobj=f).extractall(path=output_dir)


def ReadStampFile(path=STAMP_FILE):
  """Return the contents of the stamp file, or '' if it doesn't exist."""
  try:
    with open(path, 'r') as f:
      return f.read().rstrip()
  except IOError:
    return ''


def WriteStampFile(s, path=STAMP_FILE):
  """Write s to the stamp file."""
  EnsureDirExists(os.path.dirname(path))
  with open(path, 'w') as f:
    f.write(s)
    f.write('\n')


def RmTree(dir):
  """Delete dir."""
  def ChmodAndRetry(func, path, _):
    # Subversion can leave read-only files around.
    if not os.access(path, os.W_OK):
      os.chmod(path, stat.S_IWUSR)
      return func(path)
    raise

  shutil.rmtree(dir, onerror=ChmodAndRetry)


def CopyFile(src, dst):
  """Copy a file from src to dst."""
  print "Copying %s to %s" % (src, dst)
  shutil.copy(src, dst)


vs_version = None
def GetVSVersion():
  global vs_version
  if vs_version:
    return vs_version

  # Try using the toolchain in depot_tools.
  # This sets environment variables used by SelectVisualStudioVersion below.
  sys.path.append(THIS_DIR)
  import vs_toolchain
  vs_toolchain.SetEnvironmentAndGetRuntimeDllDirs()

  # Use gyp to find the MSVS installation, either in depot_tools as per above,
  # or a system-wide installation otherwise.
  sys.path.append(os.path.join(THIS_DIR, 'gyp', 'pylib'))
  import gyp.MSVSVersion
  vs_version = gyp.MSVSVersion.SelectVisualStudioVersion(
      vs_toolchain.GetVisualStudioVersion())
  return vs_version


def CopyDiaDllTo(target_dir):
  # This script always wants to use the 64-bit msdia*.dll.
  dia_path = os.path.join(GetVSVersion().Path(), 'DIA SDK', 'bin', 'amd64')
  dia_dll = os.path.join(dia_path, DIA_DLL[GetVSVersion().ShortName()])
  CopyFile(dia_dll, target_dir)


def UpdateClang():
  cds_file = "clang-%s.tgz" %  PACKAGE_VERSION
  if sys.platform == 'win32' or sys.platform == 'cygwin':
    cds_full_url = CDS_URL + '/Win/' + cds_file
  elif sys.platform.startswith('linux'):
    cds_full_url = CDS_URL + '/Linux_x64/' + cds_file
  else:
    return 0

  print 'Updating Clang to %s...' % PACKAGE_VERSION

  if ReadStampFile() == PACKAGE_VERSION:
    print 'Clang is already up to date.'
    return 0

  # Reset the stamp file in case the build is unsuccessful.
  WriteStampFile('')

  print 'Downloading prebuilt clang'
  if os.path.exists(LLVM_BUILD_DIR):
    RmTree(LLVM_BUILD_DIR)
  try:
    DownloadAndUnpack(cds_full_url, LLVM_BUILD_DIR)
    print 'clang %s unpacked' % PACKAGE_VERSION
    if sys.platform == 'win32':
      CopyDiaDllTo(os.path.join(LLVM_BUILD_DIR, 'bin'))
    WriteStampFile(PACKAGE_VERSION)
    return 0
  except urllib2.URLError:
    print 'Failed to download prebuilt clang %s' % cds_file
    print 'Exiting.'
    return 1


def main():
  # Don't buffer stdout, so that print statements are immediately flushed.
  sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
  return UpdateClang()


if __name__ == '__main__':
  sys.exit(main())



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides distutils command classes for the GRPC Python setup process."""

import distutils
import glob
import os
import os.path
import platform
import re
import shutil
import subprocess
import sys
import traceback

import setuptools
from setuptools.command import build_ext
from setuptools.command import build_py
from setuptools.command import easy_install
from setuptools.command import install
from setuptools.command import test

import support

PYTHON_STEM = os.path.dirname(os.path.abspath(__file__))
GRPC_STEM = os.path.abspath(PYTHON_STEM + '../../../../')
PROTO_STEM = os.path.join(GRPC_STEM, 'src', 'proto')
PROTO_GEN_STEM = os.path.join(GRPC_STEM, 'src', 'python', 'gens')
CYTHON_STEM = os.path.join(PYTHON_STEM, 'grpc', '_cython')


class CommandError(Exception):
    """Simple exception class for GRPC custom commands."""


# TODO(atash): Remove this once PyPI has better Linux bdist support. See
# https://bitbucket.org/pypa/pypi/issues/120/binary-wheels-for-linux-are-not-supported
def _get_grpc_custom_bdist(decorated_basename, target_bdist_basename):
    """Returns a string path to a bdist file for Linux to install.

  If we can retrieve a pre-compiled bdist from online, uses it. Else, emits a
  warning and builds from source.
  """
    # TODO(atash): somehow the name that's returned from `wheel` is different
    # between different versions of 'wheel' (but from a compatibility standpoint,
    # the names are compatible); we should have some way of determining name
    # compatibility in the same way `wheel` does to avoid having to rename all of
    # the custom wheels that we build/upload to GCS.

    # Break import style to ensure that setup.py has had a chance to install the
    # relevant package.
    from six.moves.urllib import request
    decorated_path = decorated_basename + GRPC_CUSTOM_BDIST_EXT
    try:
        url = BINARIES_REPOSITORY + '/{target}'.format(target=decorated_path)
        bdist_data = request.urlopen(url).read()
    except IOError as error:
        raise CommandError('{}\n\nCould not find the bdist {}: {}'.format(
            traceback.format_exc(), decorated_path, error.message))
    # Our chosen local bdist path.
    bdist_path = target_bdist_basename + GRPC_CUSTOM_BDIST_EXT
    try:
        with open(bdist_path, 'w') as bdist_file:
            bdist_file.write(bdist_data)
    except IOError as error:
        raise CommandError('{}\n\nCould not write grpcio bdist: {}'.format(
            traceback.format_exc(), error.message))
    return bdist_path


class SphinxDocumentation(setuptools.Command):
    """Command to generate documentation via sphinx."""

    description = 'generate sphinx documentation'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # We import here to ensure that setup.py has had a chance to install the
        # relevant package eggs first.
        import sphinx.cmd.build
        source_dir = os.path.join(GRPC_STEM, 'doc', 'python', 'sphinx')
        target_dir = os.path.join(GRPC_STEM, 'doc', 'build')
        exit_code = sphinx.cmd.build.build_main(
            ['-b', 'html', '-W', '--keep-going', source_dir, target_dir])
        if exit_code is not 0:
            raise CommandError(
                "Documentation generation has warnings or errors")


class BuildProjectMetadata(setuptools.Command):
    """Command to generate project metadata in a module."""

    description = 'build grpcio project metadata files'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        with open(os.path.join(PYTHON_STEM, 'grpc/_grpcio_metadata.py'),
                  'w') as module_file:
            module_file.write('__version__ = """{}"""'.format(
                self.distribution.get_version()))


class BuildPy(build_py.build_py):
    """Custom project build command."""

    def run(self):
        self.run_command('build_project_metadata')
        build_py.build_py.run(self)


def _poison_extensions(extensions, message):
    """Includes a file that will always fail to compile in all extensions."""
    poison_filename = os.path.join(PYTHON_STEM, 'poison.c')
    with open(poison_filename, 'w') as poison:
        poison.write('#error {}'.format(message))
    for extension in extensions:
        extension.sources = [poison_filename]


def check_and_update_cythonization(extensions):
    """Replace .pyx files with their generated counterparts and return whether or
     not cythonization still needs to occur."""
    for extension in extensions:
        generated_pyx_sources = []
        other_sources = []
        for source in extension.sources:
            base, file_ext = os.path.splitext(source)
            if file_ext == '.pyx':
                generated_pyx_source = next(
                    (base + gen_ext for gen_ext in (
                        '.c',
                        '.cpp',
                    ) if os.path.isfile(base + gen_ext)), None)
                if generated_pyx_source:
                    generated_pyx_sources.append(generated_pyx_source)
                else:
                    sys.stderr.write('Cython-generated files are missing...\n')
                    return False
            else:
                other_sources.append(source)
        extension.sources = generated_pyx_sources + other_sources
    sys.stderr.write('Found cython-generated files...\n')
    return True


def try_cythonize(extensions, linetracing=False, mandatory=True):
    """Attempt to cythonize the extensions.

  Args:
    extensions: A list of `distutils.extension.Extension`.
    linetracing: A bool indicating whether or not to enable linetracing.
    mandatory: Whether or not having Cython-generated files is mandatory. If it
      is, extensions will be poisoned when they can't be fully generated.
  """
    try:
        # Break import style to ensure we have access to Cython post-setup_requires
        import Cython.Build
    except ImportError:
        if mandatory:
            sys.stderr.write(
                "This package needs to generate C files with Cython but it cannot. "
                "Poisoning extension sources to disallow extension commands...")
            _poison_extensions(
                extensions,
                "Extensions have been poisoned due to missing Cython-generated code."
            )
        return extensions
    cython_compiler_directives = {}
    if linetracing:
        additional_define_macros = [('CYTHON_TRACE_NOGIL', '1')]
        cython_compiler_directives['linetrace'] = True
    return Cython.Build.cythonize(
        extensions,
        include_path=[
            include_dir
            for extension in extensions
            for include_dir in extension.include_dirs
        ] + [CYTHON_STEM],
        compiler_directives=cython_compiler_directives)


class BuildExt(build_ext.build_ext):
    """Custom build_ext command to enable compiler-specific flags."""

    C_OPTIONS = {
        'unix': ('-pthread',),
        'msvc': (),
    }
    LINK_OPTIONS = {}

    def build_extensions(self):

        def compiler_ok_with_extra_std():
            """Test if default compiler is okay with specifying c++ version
            when invoked in C mode. GCC is okay with this, while clang is not.
            """
            cc_test = subprocess.Popen(
                ['cc', '-x', 'c', '-std=c++11', '-'],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            _, cc_err = cc_test.communicate(input=b'int main(){return 0;}')
            return not 'invalid argument' in str(cc_err)

        # This special conditioning is here due to difference of compiler
        #   behavior in gcc and clang. The clang doesn't take --stdc++11
        #   flags but gcc does. Since the setuptools of Python only support
        #   all C or all C++ compilation, the mix of C and C++ will crash.
        #   *By default*, macOS and FreBSD use clang and Linux use gcc
        #
        #   If we are not using a permissive compiler that's OK with being
        #   passed wrong std flags, swap out compile function by adding a filter
        #   for it.
        if not compiler_ok_with_extra_std():
            old_compile = self.compiler._compile

            def new_compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
                if src[-2:] == '.c':
                    extra_postargs = [
                        arg for arg in extra_postargs if not '-std=c++' in arg
                    ]
                return old_compile(obj, src, ext, cc_args, extra_postargs,
                                   pp_opts)

            self.compiler._compile = new_compile

        compiler = self.compiler.compiler_type
        if compiler in BuildExt.C_OPTIONS:
            for extension in self.extensions:
                extension.extra_compile_args += list(
                    BuildExt.C_OPTIONS[compiler])
        if compiler in BuildExt.LINK_OPTIONS:
            for extension in self.extensions:
                extension.extra_link_args += list(
                    BuildExt.LINK_OPTIONS[compiler])
        if not check_and_update_cythonization(self.extensions):
            self.extensions = try_cythonize(self.extensions)
        try:
            build_ext.build_ext.build_extensions(self)
        except Exception as error:
            formatted_exception = traceback.format_exc()
            support.diagnose_build_ext_error(self, error, formatted_exception)
            raise CommandError(
                "Failed `build_ext` step:\n{}".format(formatted_exception))


class Gather(setuptools.Command):
    """Command to gather project dependencies."""

    description = 'gather dependencies for grpcio'
    user_options = [('test', 't',
                     'flag indicating to gather test dependencies'),
                    ('install', 'i',
                     'flag indicating to gather install dependencies')]

    def initialize_options(self):
        self.test = False
        self.install = False

    def finalize_options(self):
        # distutils requires this override.
        pass

    def run(self):
        if self.install and self.distribution.install_requires:
            self.distribution.fetch_build_eggs(
                self.distribution.install_requires)
        if self.test and self.distribution.tests_require:
            self.distribution.fetch_build_eggs(self.distribution.tests_require)



# Copyright (c) 2015, Google Inc.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""Enumerates source files for consumption by various build systems."""

import optparse
import os
import subprocess
import sys
import json


# OS_ARCH_COMBOS maps from OS and platform to the OpenSSL assembly "style" for
# that platform and the extension used by asm files.
OS_ARCH_COMBOS = [
    ('ios', 'arm', 'ios32', [], 'S'),
    ('ios', 'aarch64', 'ios64', [], 'S'),
    ('linux', 'arm', 'linux32', [], 'S'),
    ('linux', 'aarch64', 'linux64', [], 'S'),
    ('linux', 'ppc64le', 'linux64le', [], 'S'),
    ('linux', 'x86', 'elf', ['-fPIC', '-DOPENSSL_IA32_SSE2'], 'S'),
    ('linux', 'x86_64', 'elf', [], 'S'),
    ('mac', 'x86', 'macosx', ['-fPIC', '-DOPENSSL_IA32_SSE2'], 'S'),
    ('mac', 'x86_64', 'macosx', [], 'S'),
    ('win', 'x86', 'win32n', ['-DOPENSSL_IA32_SSE2'], 'asm'),
    ('win', 'x86_64', 'nasm', [], 'asm'),
]

# NON_PERL_FILES enumerates assembly files that are not processed by the
# perlasm system.
NON_PERL_FILES = {
    ('linux', 'arm'): [
        'src/crypto/curve25519/asm/x25519-asm-arm.S',
        'src/crypto/poly1305/poly1305_arm_asm.S',
    ],
}

PREFIX = None


def PathOf(x):
  return x if not PREFIX else os.path.join(PREFIX, x)


class Android(object):

  def __init__(self):
    self.header = \
"""# Copyright (C) 2015 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is created by generate_build_files.py. Do not edit manually.

"""

  def PrintVariableSection(self, out, name, files):
    out.write('%s := \\\n' % name)
    for f in sorted(files):
      out.write('  %s\\\n' % f)
    out.write('\n')

  def WriteFiles(self, files, asm_outputs):
    # New Android.bp format
    with open('sources.bp', 'w+') as blueprint:
      blueprint.write(self.header.replace('#', '//'))

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "libcrypto_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['crypto']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('    target: {\n')

      for ((osname, arch), asm_files) in asm_outputs:
        if osname != 'linux' or arch == 'ppc64le':
          continue
        if arch == 'aarch64':
          arch = 'arm64'

        blueprint.write('        linux_%s: {\n' % arch)
        blueprint.write('            srcs: [\n')
        for f in sorted(asm_files):
          blueprint.write('                "%s",\n' % f)
        blueprint.write('            ],\n')
        blueprint.write('        },\n')

      blueprint.write('    },\n')
      blueprint.write('}\n\n')

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "libssl_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['ssl']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('}\n\n')

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "bssl_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['tool']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('}\n\n')

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "boringssl_test_support_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['test_support']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('}\n\n')

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "boringssl_crypto_test_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['crypto_test']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('}\n\n')

      blueprint.write('cc_defaults {\n')
      blueprint.write('    name: "boringssl_ssl_test_sources",\n')
      blueprint.write('    srcs: [\n')
      for f in sorted(files['ssl_test']):
        blueprint.write('        "%s",\n' % f)
      blueprint.write('    ],\n')
      blueprint.write('}\n\n')

    # Legacy Android.mk format, only used by Trusty in new branches
    with open('sources.mk', 'w+') as makefile:
      makefile.write(self.header)

      self.PrintVariableSection(makefile, 'crypto_sources', files['crypto'])

      for ((osname, arch), asm_files) in asm_outputs:
        if osname != 'linux':
          continue
        self.PrintVariableSection(
            makefile, '%s_%s_sources' % (osname, arch), asm_files)


class Bazel(object):
  """Bazel outputs files suitable for including in Bazel files."""

  def __init__(self):
    self.firstSection = True
    self.header = \
"""# This file is created by generate_build_files.py. Do not edit manually.

"""

  def PrintVariableSection(self, out, name, files):
    if not self.firstSection:
      out.write('\n')
    self.firstSection = False

    out.write('%s = [\n' % name)
    for f in sorted(files):
      out.write('    "%s",\n' % PathOf(f))
    out.write(']\n')

  def WriteFiles(self, files, asm_outputs):
    with open('BUILD.generated.bzl', 'w+') as out:
      out.write(self.header)

      self.PrintVariableSection(out, 'ssl_headers', files['ssl_headers'])
      self.PrintVariableSection(out, 'fips_fragments', files['fips_fragments'])
      self.PrintVariableSection(
          out, 'ssl_internal_headers', files['ssl_internal_headers'])
      self.PrintVariableSection(out, 'ssl_sources', files['ssl'])
      self.PrintVariableSection(out, 'crypto_headers', files['crypto_headers'])
      self.PrintVariableSection(
          out, 'crypto_internal_headers', files['crypto_internal_headers'])
      self.PrintVariableSection(out, 'crypto_sources', files['crypto'])
      self.PrintVariableSection(out, 'tool_sources', files['tool'])
      self.PrintVariableSection(out, 'tool_headers', files['tool_headers'])

      for ((osname, arch), asm_files) in asm_outputs:
        self.PrintVariableSection(
            out, 'crypto_sources_%s_%s' % (osname, arch), asm_files)

    with open('BUILD.generated_tests.bzl', 'w+') as out:
      out.write(self.header)

      out.write('test_support_sources = [\n')
      for filename in sorted(files['test_support'] +
                             files['test_support_headers'] +
                             files['crypto_internal_headers'] +
                             files['ssl_internal_headers']):
        if os.path.basename(filename) == 'malloc.cc':
          continue
        out.write('    "%s",\n' % PathOf(filename))

      out.write(']\n')

      self.PrintVariableSection(out, 'crypto_test_sources',
                                files['crypto_test'])
      self.PrintVariableSection(out, 'ssl_test_sources', files['ssl_test'])


class Eureka(object):

  def __init__(self):
    self.header = \
"""# Copyright (C) 2017 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file is created by generate_build_files.py. Do not edit manually.

"""

  def PrintVariableSection(self, out, name, files):
    out.write('%s := \\\n' % name)
    for f in sorted(files):
      out.write('  %s\\\n' % f)
    out.write('\n')

  def WriteFiles(self, files, asm_outputs):
    # Legacy Android.mk format
    with open('eureka.mk', 'w+') as makefile:
      makefile.write(self.header)

      self.PrintVariableSection(makefile, 'crypto_sources', files['crypto'])
      self.PrintVariableSection(makefile, 'ssl_sources', files['ssl'])
      self.PrintVariableSection(makefile, 'tool_sources', files['tool'])

      for ((osname, arch), asm_files) in asm_outputs:
        if osname != 'linux':
          continue
        self.PrintVariableSection(
            makefile, '%s_%s_sources' % (osname, arch), asm_files)


class GN(object):

  def __init__(self):
    self.firstSection = True
    self.header = \
"""# Copyright (c) 2016 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

# This file is created by generate_build_files.py. Do not edit manually.

"""

  def PrintVariableSection(self, out, name, files):
    if not self.firstSection:
      out.write('\n')
    self.firstSection = False

    out.write('%s = [\n' % name)
    for f in sorted(files):
      out.write('  "%s",\n' % f)
    out.write(']\n')

  def WriteFiles(self, files, asm_outputs):
    with open('BUILD.generated.gni', 'w+') as out:
      out.write(self.header)

      self.PrintVariableSection(out, 'crypto_sources',
                                files['crypto'] + files['crypto_headers'] +
                                files['crypto_internal_headers'])
      self.PrintVariableSection(out, 'ssl_sources',
                                files['ssl'] + files['ssl_headers'] +
                                files['ssl_internal_headers'])

      for ((osname, arch), asm_files) in asm_outputs:
        self.PrintVariableSection(
            out, 'crypto_sources_%s_%s' % (osname, arch), asm_files)

      fuzzers = [os.path.splitext(os.path.basename(fuzzer))[0]
                 for fuzzer in files['fuzz']]
      self.PrintVariableSection(out, 'fuzzers', fuzzers)

    with open('BUILD.generated_tests.gni', 'w+') as out:
      self.firstSection = True
      out.write(self.header)

      self.PrintVariableSection(out, 'test_support_sources',
                                files['test_support'] +
                                files['test_support_headers'])
      self.PrintVariableSection(out, 'crypto_test_sources',
                                files['crypto_test'])
      self.PrintVariableSection(out, 'ssl_test_sources', files['ssl_test'])


class GYP(object):

  def __init__(self):
    self.header = \
"""# Copyright (c) 2016 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

# This file is created by generate_build_files.py. Do not edit manually.

"""

  def PrintVariableSection(self, out, name, files):
    out.write('    \'%s\': [\n' % name)
    for f in sorted(files):
      out.write('      \'%s\',\n' % f)
    out.write('    ],\n')

  def WriteFiles(self, files, asm_outputs):
    with open('boringssl.gypi', 'w+') as gypi:
      gypi.write(self.header + '{\n  \'variables\': {\n')

      self.PrintVariableSection(gypi, 'boringssl_ssl_sources',
                                files['ssl'] + files['ssl_headers'] +
                                files['ssl_internal_headers'])
      self.PrintVariableSection(gypi, 'boringssl_crypto_sources',
                                files['crypto'] + files['crypto_headers'] +
                                files['crypto_internal_headers'])

      for ((osname, arch), asm_files) in asm_outputs:
        self.PrintVariableSection(gypi, 'boringssl_%s_%s_sources' %
                                  (osname, arch), asm_files)

      gypi.write('  }\n}\n')


def FindCMakeFiles(directory):
  """Returns list of all CMakeLists.txt files recursively in directory."""
  cmakefiles = []

  for (path, _, filenames) in os.walk(directory):
    for filename in filenames:
      if filename == 'CMakeLists.txt':
        cmakefiles.append(os.path.join(path, filename))

  return cmakefiles

def OnlyFIPSFragments(path, dent, is_dir):
  return is_dir or (path.startswith(
      os.path.join('src', 'crypto', 'fipsmodule', '')) and
      NoTests(path, dent, is_dir))

def NoTestsNorFIPSFragments(path, dent, is_dir):
  return (NoTests(path, dent, is_dir) and
      (is_dir or not OnlyFIPSFragments(path, dent, is_dir)))

def NoTests(path, dent, is_dir):
  """Filter function that can be passed to FindCFiles in order to remove test
  sources."""
  if is_dir:
    return dent != 'test'
  return 'test.' not in dent


def OnlyTests(path, dent, is_dir):
  """Filter function that can be passed to FindCFiles in order to remove
  non-test sources."""
  if is_dir:
    return dent != 'test'
  return '_test.' in dent


def AllFiles(path, dent, is_dir):
  """Filter function that can be passed to FindCFiles in order to include all
  sources."""
  return True


def NoTestRunnerFiles(path, dent, is_dir):
  """Filter function that can be passed to FindCFiles or FindHeaderFiles in
  order to exclude test runner files."""
  # NOTE(martinkr): This prevents .h/.cc files in src/ssl/test/runner, which
  # are in their own subpackage, from being included in boringssl/BUILD files.
  return not is_dir or dent != 'runner'


def NotGTestSupport(path, dent, is_dir):
  return 'gtest' not in dent


def SSLHeaderFiles(path, dent, is_dir):
  return dent in ['ssl.h', 'tls1.h', 'ssl23.h', 'ssl3.h', 'dtls1.h']


def FindCFiles(directory, filter_func):
  """Recurses through directory and returns a list of paths to all the C source
  files that pass filter_func."""
  cfiles = []

  for (path, dirnames, filenames) in os.walk(directory):
    for filename in filenames:
      if not filename.endswith('.c') and not filename.endswith('.cc'):
        continue
      if not filter_func(path, filename, False):
        continue
      cfiles.append(os.path.join(path, filename))

    for (i, dirname) in enumerate(dirnames):
      if not filter_func(path, dirname, True):
        del dirnames[i]

  return cfiles


def FindHeaderFiles(directory, filter_func):
  """Recurses through directory and returns a list of paths to all the header files that pass filter_func."""
  hfiles = []

  for (path, dirnames, filenames) in os.walk(directory):
    for filename in filenames:
      if not filename.endswith('.h'):
        continue
      if not filter_func(path, filename, False):
        continue
      hfiles.append(os.path.join(path, filename))

      for (i, dirname) in enumerate(dirnames):
        if not filter_func(path, dirname, True):
          del dirnames[i]

  return hfiles


def ExtractPerlAsmFromCMakeFile(cmakefile):
  """Parses the contents of the CMakeLists.txt file passed as an argument and
  returns a list of all the perlasm() directives found in the file."""
  perlasms = []
  with open(cmakefile) as f:
    for line in f:
      line = line.strip()
      if not line.startswith('perlasm('):
        continue
      if not line.endswith(')'):
        raise ValueError('Bad perlasm line in %s' % cmakefile)
      # Remove "perlasm(" from start and ")" from end
      params = line[8:-1].split()
      if len(params) < 2:
        raise ValueError('Bad perlasm line in %s' % cmakefile)
      perlasms.append({
          'extra_args': params[2:],
          'input': os.path.join(os.path.dirname(cmakefile), params[1]),
          'output': os.path.join(os.path.dirname(cmakefile), params[0]),
      })

  return perlasms


def ReadPerlAsmOperations():
  """Returns a list of all perlasm() directives found in CMake config files in
  src/."""
  perlasms = []
  cmakefiles = FindCMakeFiles('src')

  for cmakefile in cmakefiles:
    perlasms.extend(ExtractPerlAsmFromCMakeFile(cmakefile))

  return perlasms


def PerlAsm(output_filename, input_filename, perlasm_style, extra_args):
  """Runs the a perlasm script and puts the output into output_filename."""
  base_dir = os.path.dirname(output_filename)
  if not os.path.isdir(base_dir):
    os.makedirs(base_dir)
  subprocess.check_call(
      ['perl', input_filename, perlasm_style] + extra_args + [output_filename])


def ArchForAsmFilename(filename):
  """Returns the architectures that a given asm file should be compiled for
  based on substrings in the filename."""

  if 'x86_64' in filename or 'avx2' in filename:
    return ['x86_64']
  elif ('x86' in filename and 'x86_64' not in filename) or '586' in filename:
    return ['x86']
  elif 'armx' in filename:
    return ['arm', 'aarch64']
  elif 'armv8' in filename:
    return ['aarch64']
  elif 'arm' in filename:
    return ['arm']
  elif 'ppc' in filename:
    return ['ppc64le']
  else:
    raise ValueError('Unknown arch for asm filename: ' + filename)


def WriteAsmFiles(perlasms):
  """Generates asm files from perlasm directives for each supported OS x
  platform combination."""
  asmfiles = {}

  for osarch in OS_ARCH_COMBOS:
    (osname, arch, perlasm_style, extra_args, asm_ext) = osarch
    key = (osname, arch)
    outDir = '%s-%s' % key

    for perlasm in perlasms:
      filename = os.path.basename(perlasm['input'])
      output = perlasm['output']
      if not output.startswith('src'):
        raise ValueError('output missing src: %s' % output)
      output = os.path.join(outDir, output[4:])
      if output.endswith('-armx.${ASM_EXT}'):
        output = output.replace('-armx',
                                '-armx64' if arch == 'aarch64' else '-armx32')
      output = output.replace('${ASM_EXT}', asm_ext)

      if arch in ArchForAsmFilename(filename):
        PerlAsm(output, perlasm['input'], perlasm_style,
                perlasm['extra_args'] + extra_args)
        asmfiles.setdefault(key, []).append(output)

  for (key, non_perl_asm_files) in NON_PERL_FILES.iteritems():
    asmfiles.setdefault(key, []).extend(non_perl_asm_files)

  return asmfiles


def ExtractVariablesFromCMakeFile(cmakefile):
  """Parses the contents of the CMakeLists.txt file passed as an argument and
  returns a dictionary of exported source lists."""
  variables = {}
  in_set_command = False
  set_command = []
  with open(cmakefile) as f:
    for line in f:
      if '#' in line:
        line = line[:line.index('#')]
      line = line.strip()

      if not in_set_command:
        if line.startswith('set('):
          in_set_command = True
          set_command = []
      elif line == ')':
        in_set_command = False
        if not set_command:
          raise ValueError('Empty set command')
        variables[set_command[0]] = set_command[1:]
      else:
        set_command.extend([c for c in line.split(' ') if c])

  if in_set_command:
    raise ValueError('Unfinished set command')
  return variables


def main(platforms):
  cmake = ExtractVariablesFromCMakeFile(os.path.join('src', 'sources.cmake'))
  crypto_c_files = (FindCFiles(os.path.join('src', 'crypto'), NoTestsNorFIPSFragments) +
                    FindCFiles(os.path.join('src', 'third_party', 'fiat'), NoTestsNorFIPSFragments))
  fips_fragments = FindCFiles(os.path.join('src', 'crypto', 'fipsmodule'), OnlyFIPSFragments)
  ssl_source_files = FindCFiles(os.path.join('src', 'ssl'), NoTests)
  tool_c_files = FindCFiles(os.path.join('src', 'tool'), NoTests)
  tool_h_files = FindHeaderFiles(os.path.join('src', 'tool'), AllFiles)

  # third_party/fiat/p256.c lives in third_party/fiat, but it is a FIPS
  # fragment, not a normal source file.
  p256 = os.path.join('src', 'third_party', 'fiat', 'p256.c')
  fips_fragments.append(p256)
  crypto_c_files.remove(p256)

  # Generate err_data.c
  with open('err_data.c', 'w+') as err_data:
    subprocess.check_call(['go', 'run', 'err_data_generate.go'],
                          cwd=os.path.join('src', 'crypto', 'err'),
                          stdout=err_data)
  crypto_c_files.append('err_data.c')

  test_support_c_files = FindCFiles(os.path.join('src', 'crypto', 'test'),
                                    NotGTestSupport)
  test_support_h_files = (
      FindHeaderFiles(os.path.join('src', 'crypto', 'test'), AllFiles) +
      FindHeaderFiles(os.path.join('src', 'ssl', 'test'), NoTestRunnerFiles))

  # Generate crypto_test_data.cc
  with open('crypto_test_data.cc', 'w+') as out:
    subprocess.check_call(
        ['go', 'run', 'util/embed_test_data.go'] + cmake['CRYPTO_TEST_DATA'],
        cwd='src',
        stdout=out)

  crypto_test_files = FindCFiles(os.path.join('src', 'crypto'), OnlyTests)
  crypto_test_files += [
      'crypto_test_data.cc',
      'src/crypto/test/file_test_gtest.cc',
      'src/crypto/test/gtest_main.cc',
  ]

  ssl_test_files = FindCFiles(os.path.join('src', 'ssl'), OnlyTests)
  ssl_test_files.append('src/crypto/test/gtest_main.cc')

  fuzz_c_files = FindCFiles(os.path.join('src', 'fuzz'), NoTests)

  ssl_h_files = (
      FindHeaderFiles(
          os.path.join('src', 'include', 'openssl'),
          SSLHeaderFiles))

  def NotSSLHeaderFiles(path, filename, is_dir):
    return not SSLHeaderFiles(path, filename, is_dir)
  crypto_h_files = (
      FindHeaderFiles(
          os.path.join('src', 'include', 'openssl'),
          NotSSLHeaderFiles))

  ssl_internal_h_files = FindHeaderFiles(os.path.join('src', 'ssl'), NoTests)
  crypto_internal_h_files = (
      FindHeaderFiles(os.path.join('src', 'crypto'), NoTests) +
      FindHeaderFiles(os.path.join('src', 'third_party', 'fiat'), NoTests))

  files = {
      'crypto': crypto_c_files,
      'crypto_headers': crypto_h_files,
      'crypto_internal_headers': crypto_internal_h_files,
      'crypto_test': sorted(crypto_test_files),
      'fips_fragments': fips_fragments,
      'fuzz': fuzz_c_files,
      'ssl': ssl_source_files,
      'ssl_headers': ssl_h_files,
      'ssl_internal_headers': ssl_internal_h_files,
      'ssl_test': sorted(ssl_test_files),
      'tool': tool_c_files,
      'tool_headers': tool_h_files,
      'test_support': test_support_c_files,
      'test_support_headers': test_support_h_files,
  }

  asm_outputs = sorted(WriteAsmFiles(ReadPerlAsmOperations()).iteritems())

  for platform in platforms:
    platform.WriteFiles(files, asm_outputs)

  return 0


if __name__ == '__main__':
  parser = optparse.OptionParser(usage='Usage: %prog [--prefix=<path>]'
      ' [android|bazel|eureka|gn|gyp]')
  parser.add_option('--prefix', dest='prefix',
      help='For Bazel, prepend argument to all source files')
  options, args = parser.parse_args(sys.argv[1:])
  PREFIX = options.prefix

  if not args:
    parser.print_help()
    sys.exit(1)

  platforms = []
  for s in args:
    if s == 'android':
      platforms.append(Android())
    elif s == 'bazel':
      platforms.append(Bazel())
    elif s == 'eureka':
      platforms.append(Eureka())
    elif s == 'gn':
      platforms.append(GN())
    elif s == 'gyp':
      platforms.append(GYP())
    else:
      parser.print_help()
      sys.exit(1)

  sys.exit(main(platforms))



#!/usr/bin/env python
# Copyright 2014 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

# Modified from go/env.py in Chromium infrastructure's repository to patch out
# everything but the core toolchain.
#
# https://chromium.googlesource.com/infra/infra/

"""Can be used to point environment variable to hermetic Go toolset.

Usage (on linux and mac):
$ eval `./env.py`
$ go version

Or it can be used to wrap a command:

$ ./env.py go version
"""

assert __name__ == '__main__'

import imp
import os
import subprocess
import sys

# Do not want to mess with sys.path, load the module directly.
bootstrap = imp.load_source(
    'bootstrap', os.path.join(os.path.dirname(__file__), 'bootstrap.py'))

old = os.environ.copy()
new = bootstrap.prepare_go_environ()

if len(sys.argv) == 1:
  for key, value in sorted(new.iteritems()):
    if old.get(key) != value:
      print 'export %s="%s"' % (key, value)
else:
  exe = sys.argv[1]
  if exe == 'python':
    exe = sys.executable
  else:
    # Help Windows to find the executable in new PATH, do it only when
    # executable is referenced by name (and not by path).
    if os.sep not in exe:
      exe = bootstrap.find_executable(exe, [bootstrap.WORKSPACE])
  sys.exit(subprocess.call([exe] + sys.argv[2:], env=new))



# Copyright (c) 2015, Google Inc.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import subprocess
import sys

import vs_toolchain
# vs_toolchain adds gyp to sys.path.
import gyp.MSVSVersion

if len(sys.argv) < 2:
  print >>sys.stderr, "Usage: vs_env.py TARGET_ARCH CMD..."
  sys.exit(1)

target_arch = sys.argv[1]
cmd = sys.argv[2:]

vs_toolchain.SetEnvironmentAndGetRuntimeDllDirs()
vs_version = gyp.MSVSVersion.SelectVisualStudioVersion()

# Using shell=True is somewhat ugly, but the alternative is to pull in a copy
# of the Chromium GN build's setup_toolchain.py which runs the setup script,
# then 'set', and then parses the environment variables out. (GYP internally
# does the same thing.)
sys.exit(subprocess.call(vs_version.SetupScript(target_arch) + ["&&"] + cmd,
                         shell=True))



# Copyright 2014 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

import json
import os
import pipes
import shutil
import subprocess
import sys


script_dir = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, os.path.join(script_dir, 'gyp', 'pylib'))
json_data_file = os.path.join(script_dir, 'win_toolchain.json')


import gyp


# Use MSVS2015 as the default toolchain.
CURRENT_DEFAULT_TOOLCHAIN_VERSION = '2015'


def SetEnvironmentAndGetRuntimeDllDirs():
  """Sets up os.environ to use the depot_tools VS toolchain with gyp, and
  returns the location of the VS runtime DLLs so they can be copied into
  the output directory after gyp generation.
  """
  vs_runtime_dll_dirs = None
  depot_tools_win_toolchain = \
      bool(int(os.environ.get('DEPOT_TOOLS_WIN_TOOLCHAIN', '1')))
  if sys.platform in ('win32', 'cygwin') and depot_tools_win_toolchain:
    if not os.path.exists(json_data_file):
      Update()
    with open(json_data_file, 'r') as tempf:
      toolchain_data = json.load(tempf)

    toolchain = toolchain_data['path']
    version = toolchain_data['version']
    win_sdk = toolchain_data.get('win_sdk')
    if not win_sdk:
      win_sdk = toolchain_data['win8sdk']
    wdk = toolchain_data['wdk']
    # TODO(scottmg): The order unfortunately matters in these. They should be
    # split into separate keys for x86 and x64. (See CopyVsRuntimeDlls call
    # below). http://crbug.com/345992
    vs_runtime_dll_dirs = toolchain_data['runtime_dirs']

    os.environ['GYP_MSVS_OVERRIDE_PATH'] = toolchain
    os.environ['GYP_MSVS_VERSION'] = version
    # We need to make sure windows_sdk_path is set to the automated
    # toolchain values in GYP_DEFINES, but don't want to override any
    # otheroptions.express
    # values there.
    gyp_defines_dict = gyp.NameValueListToDict(gyp.ShlexEnv('GYP_DEFINES'))
    gyp_defines_dict['windows_sdk_path'] = win_sdk
    os.environ['GYP_DEFINES'] = ' '.join('%s=%s' % (k, pipes.quote(str(v)))
        for k, v in gyp_defines_dict.iteritems())
    os.environ['WINDOWSSDKDIR'] = win_sdk
    os.environ['WDK_DIR'] = wdk
    # Include the VS runtime in the PATH in case it's not machine-installed.
    runtime_path = ';'.join(vs_runtime_dll_dirs)
    os.environ['PATH'] = runtime_path + ';' + os.environ['PATH']
  return vs_runtime_dll_dirs


def FindDepotTools():
  """Returns the path to depot_tools in $PATH."""
  for path in os.environ['PATH'].split(os.pathsep):
    if os.path.isfile(os.path.join(path, 'gclient.py')):
      return path
  raise Exception("depot_tools not found!")


def GetVisualStudioVersion():
  """Return GYP_MSVS_VERSION of Visual Studio.
  """
  return os.environ.get('GYP_MSVS_VERSION', CURRENT_DEFAULT_TOOLCHAIN_VERSION)


def _GetDesiredVsToolchainHashes():
  """Load a list of SHA1s corresponding to the toolchains that we want installed
  to build with."""
  env_version = GetVisualStudioVersion()
  if env_version == '2015':
    # Update 3 final with 10.0.15063.468 SDK and no vctip.exe.
    return ['f53e4598951162bad6330f7a167486c7ae5db1e5']
  if env_version == '2017':
    # VS 2017 Update 3.2 with 10.0.15063.468 SDK, patched setenv.cmd, and
    # 10.0.16299.15 debuggers.
    return ['1180cb75833ea365097e279efb2d5d7a42dee4b0']
  raise Exception('Unsupported VS version %s' % env_version)


def Update():
  """Requests an update of the toolchain to the specific hashes we have at
  this revision. The update outputs a .json of the various configuration
  information required to pass to gyp which we use in |GetToolchainDir()|.
  """
  depot_tools_win_toolchain = \
      bool(int(os.environ.get('DEPOT_TOOLS_WIN_TOOLCHAIN', '1')))
  if sys.platform in ('win32', 'cygwin') and depot_tools_win_toolchain:
    depot_tools_path = FindDepotTools()
    # Necessary so that get_toolchain_if_necessary.py will put the VS toolkit
    # in the correct directory.
    os.environ['GYP_MSVS_VERSION'] = GetVisualStudioVersion()
    get_toolchain_args = [
        sys.executable,
        os.path.join(depot_tools_path,
                    'win_toolchain',
                    'get_toolchain_if_necessary.py'),
        '--output-json', json_data_file,
      ] + _GetDesiredVsToolchainHashes()
    subprocess.check_call(get_toolchain_args)

  return 0


def main():
  if not sys.platform.startswith(('win32', 'cygwin')):
    return 0
  commands = {
      'update': Update,
  }
  if len(sys.argv) < 2 or sys.argv[1] not in commands:
    print >>sys.stderr, 'Expected one of: %s' % ', '.join(commands)
    return 1
  return commands[sys.argv[1]](*sys.argv[2:])


if __name__ == '__main__':
  sys.exit(main())



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# AUTO-GENERATED FROM `$REPO_ROOT/templates/src/python/grpcio/grpc_core_dependencies.py.template`!!!

CORE_SOURCE_FILES = [
    'third_party/address_sorting/address_sorting.c',
    'third_party/address_sorting/address_sorting_posix.c',
    'third_party/address_sorting/address_sorting_windows.c',
    'src/core/lib/gpr/alloc.cc',
    'src/core/lib/gpr/atm.cc',
    'src/core/lib/gpr/cpu_iphone.cc',
    'src/core/lib/gpr/cpu_linux.cc',
    'src/core/lib/gpr/cpu_posix.cc',
    'src/core/lib/gpr/cpu_windows.cc',
    'src/core/lib/gpr/env_linux.cc',
    'src/core/lib/gpr/env_posix.cc',
    'src/core/lib/gpr/env_windows.cc',
    'src/core/lib/gpr/host_port.cc',
    'src/core/lib/gpr/log.cc',
    'src/core/lib/gpr/log_android.cc',
    'src/core/lib/gpr/log_linux.cc',
    'src/core/lib/gpr/log_posix.cc',
    'src/core/lib/gpr/log_windows.cc',
    'src/core/lib/gpr/mpscq.cc',
    'src/core/lib/gpr/murmur_hash.cc',
    'src/core/lib/gpr/string.cc',
    'src/core/lib/gpr/string_posix.cc',
    'src/core/lib/gpr/string_util_windows.cc',
    'src/core/lib/gpr/string_windows.cc',
    'src/core/lib/gpr/sync.cc',
    'src/core/lib/gpr/sync_posix.cc',
    'src/core/lib/gpr/sync_windows.cc',
    'src/core/lib/gpr/time.cc',
    'src/core/lib/gpr/time_posix.cc',
    'src/core/lib/gpr/time_precise.cc',
    'src/core/lib/gpr/time_windows.cc',
    'src/core/lib/gpr/tls_pthread.cc',
    'src/core/lib/gpr/tmpfile_msys.cc',
    'src/core/lib/gpr/tmpfile_posix.cc',
    'src/core/lib/gpr/tmpfile_windows.cc',
    'src/core/lib/gpr/wrap_memcpy.cc',
    'src/core/lib/gprpp/arena.cc',
    'src/core/lib/gprpp/fork.cc',
    'src/core/lib/gprpp/global_config_env.cc',
    'src/core/lib/gprpp/thd_posix.cc',
    'src/core/lib/gprpp/thd_windows.cc',
    'src/core/lib/profiling/basic_timers.cc',
    'src/core/lib/profiling/stap_timers.cc',
    'src/core/lib/surface/init.cc',
    'src/core/lib/avl/avl.cc',
    'src/core/lib/backoff/backoff.cc',
    'src/core/lib/channel/channel_args.cc',
    'src/core/lib/channel/channel_stack.cc',
    'src/core/lib/channel/channel_stack_builder.cc',
    'src/core/lib/channel/channel_trace.cc',
    'src/core/lib/channel/channelz.cc',
    'src/core/lib/channel/channelz_registry.cc',
    'src/core/lib/channel/connected_channel.cc',
    'src/core/lib/channel/handshaker.cc',
    'src/core/lib/channel/handshaker_registry.cc',
    'src/core/lib/channel/status_util.cc',
    'src/core/lib/compression/compression.cc',
    'src/core/lib/compression/compression_args.cc',
    'src/core/lib/compression/compression_internal.cc',
    'src/core/lib/compression/message_compress.cc',
    'src/core/lib/compression/stream_compression.cc',
    'src/core/lib/compression/stream_compression_gzip.cc',
    'src/core/lib/compression/stream_compression_identity.cc',
    'src/core/lib/debug/stats.cc',
    'src/core/lib/debug/stats_data.cc',
    'src/core/lib/http/format_request.cc',
    'src/core/lib/http/httpcli.cc',
    'src/core/lib/http/parser.cc',
    'src/core/lib/iomgr/buffer_list.cc',
    'src/core/lib/iomgr/call_combiner.cc',
    'src/core/lib/iomgr/cfstream_handle.cc',
    'src/core/lib/iomgr/combiner.cc',
    'src/core/lib/iomgr/endpoint.cc',
    'src/core/lib/iomgr/endpoint_cfstream.cc',
    'src/core/lib/iomgr/endpoint_pair_posix.cc',
    'src/core/lib/iomgr/endpoint_pair_uv.cc',
    'src/core/lib/iomgr/endpoint_pair_windows.cc',
    'src/core/lib/iomgr/error.cc',
    'src/core/lib/iomgr/error_cfstream.cc',
    'src/core/lib/iomgr/ev_epoll1_linux.cc',
    'src/core/lib/iomgr/ev_epollex_linux.cc',
    'src/core/lib/iomgr/ev_poll_posix.cc',
    'src/core/lib/iomgr/ev_posix.cc',
    'src/core/lib/iomgr/ev_windows.cc',
    'src/core/lib/iomgr/exec_ctx.cc',
    'src/core/lib/iomgr/executor.cc',
    'src/core/lib/iomgr/fork_posix.cc',
    'src/core/lib/iomgr/fork_windows.cc',
    'src/core/lib/iomgr/gethostname_fallback.cc',
    'src/core/lib/iomgr/gethostname_host_name_max.cc',
    'src/core/lib/iomgr/gethostname_sysconf.cc',
    'src/core/lib/iomgr/grpc_if_nametoindex_posix.cc',
    'src/core/lib/iomgr/grpc_if_nametoindex_unsupported.cc',
    'src/core/lib/iomgr/internal_errqueue.cc',
    'src/core/lib/iomgr/iocp_windows.cc',
    'src/core/lib/iomgr/iomgr.cc',
    'src/core/lib/iomgr/iomgr_custom.cc',
    'src/core/lib/iomgr/iomgr_internal.cc',
    'src/core/lib/iomgr/iomgr_posix.cc',
    'src/core/lib/iomgr/iomgr_posix_cfstream.cc',
    'src/core/lib/iomgr/iomgr_uv.cc',
    'src/core/lib/iomgr/iomgr_windows.cc',
    'src/core/lib/iomgr/is_epollexclusive_available.cc',
    'src/core/lib/iomgr/load_file.cc',
    'src/core/lib/iomgr/lockfree_event.cc',
    'src/core/lib/iomgr/polling_entity.cc',
    'src/core/lib/iomgr/pollset.cc',
    'src/core/lib/iomgr/pollset_custom.cc',
    'src/core/lib/iomgr/pollset_set.cc',
    'src/core/lib/iomgr/pollset_set_custom.cc',
    'src/core/lib/iomgr/pollset_set_windows.cc',
    'src/core/lib/iomgr/pollset_uv.cc',
    'src/core/lib/iomgr/pollset_windows.cc',
    'src/core/lib/iomgr/resolve_address.cc',
    'src/core/lib/iomgr/resolve_address_custom.cc',
    'src/core/lib/iomgr/resolve_address_posix.cc',
    'src/core/lib/iomgr/resolve_address_windows.cc',
    'src/core/lib/iomgr/resource_quota.cc',
    'src/core/lib/iomgr/sockaddr_utils.cc',
    'src/core/lib/iomgr/socket_factory_posix.cc',
    'src/core/lib/iomgr/socket_mutator.cc',
    'src/core/lib/iomgr/socket_utils_common_posix.cc',
    'src/core/lib/iomgr/socket_utils_linux.cc',
    'src/core/lib/iomgr/socket_utils_posix.cc',
    'src/core/lib/iomgr/socket_utils_uv.cc',
    'src/core/lib/iomgr/socket_utils_windows.cc',
    'src/core/lib/iomgr/socket_windows.cc',
    'src/core/lib/iomgr/tcp_client.cc',
    'src/core/lib/iomgr/tcp_client_cfstream.cc',
    'src/core/lib/iomgr/tcp_client_custom.cc',
    'src/core/lib/iomgr/tcp_client_posix.cc',
    'src/core/lib/iomgr/tcp_client_windows.cc',
    'src/core/lib/iomgr/tcp_custom.cc',
    'src/core/lib/iomgr/tcp_posix.cc',
    'src/core/lib/iomgr/tcp_server.cc',
    'src/core/lib/iomgr/tcp_server_custom.cc',
    'src/core/lib/iomgr/tcp_server_posix.cc',
    'src/core/lib/iomgr/tcp_server_utils_posix_common.cc',
    'src/core/lib/iomgr/tcp_server_utils_posix_ifaddrs.cc',
    'src/core/lib/iomgr/tcp_server_utils_posix_noifaddrs.cc',
    'src/core/lib/iomgr/tcp_server_windows.cc',
    'src/core/lib/iomgr/tcp_uv.cc',
    'src/core/lib/iomgr/tcp_windows.cc',
    'src/core/lib/iomgr/time_averaged_stats.cc',
    'src/core/lib/iomgr/timer.cc',
    'src/core/lib/iomgr/timer_custom.cc',
    'src/core/lib/iomgr/timer_generic.cc',
    'src/core/lib/iomgr/timer_heap.cc',
    'src/core/lib/iomgr/timer_manager.cc',
    'src/core/lib/iomgr/timer_uv.cc',
    'src/core/lib/iomgr/udp_server.cc',
    'src/core/lib/iomgr/unix_sockets_posix.cc',
    'src/core/lib/iomgr/unix_sockets_posix_noop.cc',
    'src/core/lib/iomgr/wakeup_fd_eventfd.cc',
    'src/core/lib/iomgr/wakeup_fd_nospecial.cc',
    'src/core/lib/iomgr/wakeup_fd_pipe.cc',
    'src/core/lib/iomgr/wakeup_fd_posix.cc',
    'src/core/lib/json/json.cc',
    'src/core/lib/json/json_reader.cc',
    'src/core/lib/json/json_string.cc',
    'src/core/lib/json/json_writer.cc',
    'src/core/lib/slice/b64.cc',
    'src/core/lib/slice/percent_encoding.cc',
    'src/core/lib/slice/slice.cc',
    'src/core/lib/slice/slice_buffer.cc',
    'src/core/lib/slice/slice_intern.cc',
    'src/core/lib/slice/slice_string_helpers.cc',
    'src/core/lib/surface/api_trace.cc',
    'src/core/lib/surface/byte_buffer.cc',
    'src/core/lib/surface/byte_buffer_reader.cc',
    'src/core/lib/surface/call.cc',
    'src/core/lib/surface/call_details.cc',
    'src/core/lib/surface/call_log_batch.cc',
    'src/core/lib/surface/channel.cc',
    'src/core/lib/surface/channel_init.cc',
    'src/core/lib/surface/channel_ping.cc',
    'src/core/lib/surface/channel_stack_type.cc',
    'src/core/lib/surface/completion_queue.cc',
    'src/core/lib/surface/completion_queue_factory.cc',
    'src/core/lib/surface/event_string.cc',
    'src/core/lib/surface/lame_client.cc',
    'src/core/lib/surface/metadata_array.cc',
    'src/core/lib/surface/server.cc',
    'src/core/lib/surface/validate_metadata.cc',
    'src/core/lib/surface/version.cc',
    'src/core/lib/transport/bdp_estimator.cc',
    'src/core/lib/transport/byte_stream.cc',
    'src/core/lib/transport/connectivity_state.cc',
    'src/core/lib/transport/error_utils.cc',
    'src/core/lib/transport/metadata.cc',
    'src/core/lib/transport/metadata_batch.cc',
    'src/core/lib/transport/pid_controller.cc',
    'src/core/lib/transport/static_metadata.cc',
    'src/core/lib/transport/status_conversion.cc',
    'src/core/lib/transport/status_metadata.cc',
    'src/core/lib/transport/timeout_encoding.cc',
    'src/core/lib/transport/transport.cc',
    'src/core/lib/transport/transport_op_string.cc',
    'src/core/lib/uri/uri_parser.cc',
    'src/core/lib/debug/trace.cc',
    'src/core/ext/transport/chttp2/server/secure/server_secure_chttp2.cc',
    'src/core/ext/transport/chttp2/transport/bin_decoder.cc',
    'src/core/ext/transport/chttp2/transport/bin_encoder.cc',
    'src/core/ext/transport/chttp2/transport/chttp2_plugin.cc',
    'src/core/ext/transport/chttp2/transport/chttp2_transport.cc',
    'src/core/ext/transport/chttp2/transport/context_list.cc',
    'src/core/ext/transport/chttp2/transport/flow_control.cc',
    'src/core/ext/transport/chttp2/transport/frame_data.cc',
    'src/core/ext/transport/chttp2/transport/frame_goaway.cc',
    'src/core/ext/transport/chttp2/transport/frame_ping.cc',
    'src/core/ext/transport/chttp2/transport/frame_rst_stream.cc',
    'src/core/ext/transport/chttp2/transport/frame_settings.cc',
    'src/core/ext/transport/chttp2/transport/frame_window_update.cc',
    'src/core/ext/transport/chttp2/transport/hpack_encoder.cc',
    'src/core/ext/transport/chttp2/transport/hpack_parser.cc',
    'src/core/ext/transport/chttp2/transport/hpack_table.cc',
    'src/core/ext/transport/chttp2/transport/http2_settings.cc',
    'src/core/ext/transport/chttp2/transport/huffsyms.cc',
    'src/core/ext/transport/chttp2/transport/incoming_metadata.cc',
    'src/core/ext/transport/chttp2/transport/parsing.cc',
    'src/core/ext/transport/chttp2/transport/stream_lists.cc',
    'src/core/ext/transport/chttp2/transport/stream_map.cc',
    'src/core/ext/transport/chttp2/transport/varint.cc',
    'src/core/ext/transport/chttp2/transport/writing.cc',
    'src/core/ext/transport/chttp2/alpn/alpn.cc',
    'src/core/ext/filters/http/client/http_client_filter.cc',
    'src/core/ext/filters/http/http_filters_plugin.cc',
    'src/core/ext/filters/http/message_compress/message_compress_filter.cc',
    'src/core/ext/filters/http/server/http_server_filter.cc',
    'src/core/lib/http/httpcli_security_connector.cc',
    'src/core/lib/security/context/security_context.cc',
    'src/core/lib/security/credentials/alts/alts_credentials.cc',
    'src/core/lib/security/credentials/composite/composite_credentials.cc',
    'src/core/lib/security/credentials/credentials.cc',
    'src/core/lib/security/credentials/credentials_metadata.cc',
    'src/core/lib/security/credentials/fake/fake_credentials.cc',
    'src/core/lib/security/credentials/google_default/credentials_generic.cc',
    'src/core/lib/security/credentials/google_default/google_default_credentials.cc',
    'src/core/lib/security/credentials/iam/iam_credentials.cc',
    'src/core/lib/security/credentials/jwt/json_token.cc',
    'src/core/lib/security/credentials/jwt/jwt_credentials.cc',
    'src/core/lib/security/credentials/jwt/jwt_verifier.cc',
    'src/core/lib/security/credentials/local/local_credentials.cc',
    'src/core/lib/security/credentials/oauth2/oauth2_credentials.cc',
    'src/core/lib/security/credentials/plugin/plugin_credentials.cc',
    'src/core/lib/security/credentials/ssl/ssl_credentials.cc',
    'src/core/lib/security/credentials/tls/grpc_tls_credentials_options.cc',
    'src/core/lib/security/credentials/tls/spiffe_credentials.cc',
    'src/core/lib/security/security_connector/alts/alts_security_connector.cc',
    'src/core/lib/security/security_connector/fake/fake_security_connector.cc',
    'src/core/lib/security/security_connector/load_system_roots_fallback.cc',
    'src/core/lib/security/security_connector/load_system_roots_linux.cc',
    'src/core/lib/security/security_connector/local/local_security_connector.cc',
    'src/core/lib/security/security_connector/security_connector.cc',
    'src/core/lib/security/security_connector/ssl/ssl_security_connector.cc',
    'src/core/lib/security/security_connector/ssl_utils.cc',
    'src/core/lib/security/security_connector/tls/spiffe_security_connector.cc',
    'src/core/lib/security/transport/client_auth_filter.cc',
    'src/core/lib/security/transport/secure_endpoint.cc',
    'src/core/lib/security/transport/security_handshaker.cc',
    'src/core/lib/security/transport/server_auth_filter.cc',
    'src/core/lib/security/transport/target_authority_table.cc',
    'src/core/lib/security/transport/tsi_error.cc',
    'src/core/lib/security/util/json_util.cc',
    'src/core/lib/surface/init_secure.cc',
    'src/core/tsi/alts/crypt/aes_gcm.cc',
    'src/core/tsi/alts/crypt/gsec.cc',
    'src/core/tsi/alts/frame_protector/alts_counter.cc',
    'src/core/tsi/alts/frame_protector/alts_crypter.cc',
    'src/core/tsi/alts/frame_protector/alts_frame_protector.cc',
    'src/core/tsi/alts/frame_protector/alts_record_protocol_crypter_common.cc',
    'src/core/tsi/alts/frame_protector/alts_seal_privacy_integrity_crypter.cc',
    'src/core/tsi/alts/frame_protector/alts_unseal_privacy_integrity_crypter.cc',
    'src/core/tsi/alts/frame_protector/frame_handler.cc',
    'src/core/tsi/alts/handshaker/alts_handshaker_client.cc',
    'src/core/tsi/alts/handshaker/alts_shared_resource.cc',
    'src/core/tsi/alts/handshaker/alts_tsi_handshaker.cc',
    'src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_integrity_only_record_protocol.cc',
    'src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_privacy_integrity_record_protocol.cc',
    'src/core/tsi/alts/zero_copy_frame_protector/alts_grpc_record_protocol_common.cc',
    'src/core/tsi/alts/zero_copy_frame_protector/alts_iovec_record_protocol.cc',
    'src/core/tsi/alts/zero_copy_frame_protector/alts_zero_copy_grpc_protector.cc',
    'src/core/lib/security/credentials/alts/check_gcp_environment.cc',
    'src/core/lib/security/credentials/alts/check_gcp_environment_linux.cc',
    'src/core/lib/security/credentials/alts/check_gcp_environment_no_op.cc',
    'src/core/lib/security/credentials/alts/check_gcp_environment_windows.cc',
    'src/core/lib/security/credentials/alts/grpc_alts_credentials_client_options.cc',
    'src/core/lib/security/credentials/alts/grpc_alts_credentials_options.cc',
    'src/core/lib/security/credentials/alts/grpc_alts_credentials_server_options.cc',
    'src/core/tsi/alts/handshaker/alts_handshaker_service_api.cc',
    'src/core/tsi/alts/handshaker/alts_handshaker_service_api_util.cc',
    'src/core/tsi/alts/handshaker/alts_tsi_utils.cc',
    'src/core/tsi/alts/handshaker/transport_security_common_api.cc',
    'src/core/tsi/alts/handshaker/altscontext.pb.c',
    'src/core/tsi/alts/handshaker/handshaker.pb.c',
    'src/core/tsi/alts/handshaker/transport_security_common.pb.c',
    'third_party/nanopb/pb_common.c',
    'third_party/nanopb/pb_decode.c',
    'third_party/nanopb/pb_encode.c',
    'src/core/tsi/transport_security.cc',
    'src/core/ext/transport/chttp2/client/insecure/channel_create.cc',
    'src/core/ext/transport/chttp2/client/insecure/channel_create_posix.cc',
    'src/core/ext/transport/chttp2/client/authority.cc',
    'src/core/ext/transport/chttp2/client/chttp2_connector.cc',
    'src/core/ext/filters/client_channel/backup_poller.cc',
    'src/core/ext/filters/client_channel/channel_connectivity.cc',
    'src/core/ext/filters/client_channel/client_channel.cc',
    'src/core/ext/filters/client_channel/client_channel_channelz.cc',
    'src/core/ext/filters/client_channel/client_channel_factory.cc',
    'src/core/ext/filters/client_channel/client_channel_plugin.cc',
    'src/core/ext/filters/client_channel/connector.cc',
    'src/core/ext/filters/client_channel/global_subchannel_pool.cc',
    'src/core/ext/filters/client_channel/health/health_check_client.cc',
    'src/core/ext/filters/client_channel/http_connect_handshaker.cc',
    'src/core/ext/filters/client_channel/http_proxy.cc',
    'src/core/ext/filters/client_channel/lb_policy.cc',
    'src/core/ext/filters/client_channel/lb_policy_registry.cc',
    'src/core/ext/filters/client_channel/local_subchannel_pool.cc',
    'src/core/ext/filters/client_channel/parse_address.cc',
    'src/core/ext/filters/client_channel/proxy_mapper.cc',
    'src/core/ext/filters/client_channel/proxy_mapper_registry.cc',
    'src/core/ext/filters/client_channel/resolver.cc',
    'src/core/ext/filters/client_channel/resolver_registry.cc',
    'src/core/ext/filters/client_channel/resolver_result_parsing.cc',
    'src/core/ext/filters/client_channel/resolving_lb_policy.cc',
    'src/core/ext/filters/client_channel/retry_throttle.cc',
    'src/core/ext/filters/client_channel/server_address.cc',
    'src/core/ext/filters/client_channel/service_config.cc',
    'src/core/ext/filters/client_channel/subchannel.cc',
    'src/core/ext/filters/client_channel/subchannel_pool_interface.cc',
    'src/core/ext/filters/deadline/deadline_filter.cc',
    'src/core/ext/filters/client_channel/health/health.pb.c',
    'src/core/tsi/fake_transport_security.cc',
    'src/core/tsi/local_transport_security.cc',
    'src/core/tsi/ssl/session_cache/ssl_session_boringssl.cc',
    'src/core/tsi/ssl/session_cache/ssl_session_cache.cc',
    'src/core/tsi/ssl/session_cache/ssl_session_openssl.cc',
    'src/core/tsi/ssl_transport_security.cc',
    'src/core/tsi/transport_security_grpc.cc',
    'src/core/ext/transport/chttp2/server/chttp2_server.cc',
    'src/core/ext/transport/chttp2/client/secure/secure_channel_create.cc',
    'src/core/ext/transport/chttp2/server/insecure/server_chttp2.cc',
    'src/core/ext/transport/chttp2/server/insecure/server_chttp2_posix.cc',
    'src/core/ext/transport/inproc/inproc_plugin.cc',
    'src/core/ext/transport/inproc/inproc_transport.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/client_load_reporting_filter.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb_channel_secure.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/grpclb_client_stats.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/load_balancer_api.cc',
    'src/core/ext/filters/client_channel/resolver/fake/fake_resolver.cc',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/google/protobuf/duration.pb.c',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/google/protobuf/timestamp.pb.c',
    'src/core/ext/filters/client_channel/lb_policy/grpclb/proto/grpc/lb/v1/load_balancer.pb.c',
    'src/core/ext/filters/client_channel/lb_policy/xds/xds.cc',
    'src/core/ext/filters/client_channel/lb_policy/xds/xds_channel_secure.cc',
    'src/core/ext/filters/client_channel/lb_policy/xds/xds_client_stats.cc',
    'src/core/ext/filters/client_channel/lb_policy/xds/xds_load_balancer_api.cc',
    'src/core/ext/filters/client_channel/lb_policy/pick_first/pick_first.cc',
    'src/core/ext/filters/client_channel/lb_policy/round_robin/round_robin.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/dns_resolver_ares.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_libuv.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_posix.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_ev_driver_windows.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_fallback.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_libuv.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_posix.cc',
    'src/core/ext/filters/client_channel/resolver/dns/c_ares/grpc_ares_wrapper_windows.cc',
    'src/core/ext/filters/client_channel/resolver/dns/dns_resolver_selection.cc',
    'src/core/ext/filters/client_channel/resolver/dns/native/dns_resolver.cc',
    'src/core/ext/filters/client_channel/resolver/sockaddr/sockaddr_resolver.cc',
    'src/core/ext/filters/census/grpc_context.cc',
    'src/core/ext/filters/max_age/max_age_filter.cc',
    'src/core/ext/filters/message_size/message_size_filter.cc',
    'src/core/ext/filters/http/client_authority_filter.cc',
    'src/core/ext/filters/workarounds/workaround_cronet_compression_filter.cc',
    'src/core/ext/filters/workarounds/workaround_utils.cc',
    'src/core/plugin_registry/grpc_plugin_registry.cc',
    'src/boringssl/err_data.c',
    'third_party/boringssl/crypto/asn1/a_bitstr.c',
    'third_party/boringssl/crypto/asn1/a_bool.c',
    'third_party/boringssl/crypto/asn1/a_d2i_fp.c',
    'third_party/boringssl/crypto/asn1/a_dup.c',
    'third_party/boringssl/crypto/asn1/a_enum.c',
    'third_party/boringssl/crypto/asn1/a_gentm.c',
    'third_party/boringssl/crypto/asn1/a_i2d_fp.c',
    'third_party/boringssl/crypto/asn1/a_int.c',
    'third_party/boringssl/crypto/asn1/a_mbstr.c',
    'third_party/boringssl/crypto/asn1/a_object.c',
    'third_party/boringssl/crypto/asn1/a_octet.c',
    'third_party/boringssl/crypto/asn1/a_print.c',
    'third_party/boringssl/crypto/asn1/a_strnid.c',
    'third_party/boringssl/crypto/asn1/a_time.c',
    'third_party/boringssl/crypto/asn1/a_type.c',
    'third_party/boringssl/crypto/asn1/a_utctm.c',
    'third_party/boringssl/crypto/asn1/a_utf8.c',
    'third_party/boringssl/crypto/asn1/asn1_lib.c',
    'third_party/boringssl/crypto/asn1/asn1_par.c',
    'third_party/boringssl/crypto/asn1/asn_pack.c',
    'third_party/boringssl/crypto/asn1/f_enum.c',
    'third_party/boringssl/crypto/asn1/f_int.c',
    'third_party/boringssl/crypto/asn1/f_string.c',
    'third_party/boringssl/crypto/asn1/tasn_dec.c',
    'third_party/boringssl/crypto/asn1/tasn_enc.c',
    'third_party/boringssl/crypto/asn1/tasn_fre.c',
    'third_party/boringssl/crypto/asn1/tasn_new.c',
    'third_party/boringssl/crypto/asn1/tasn_typ.c',
    'third_party/boringssl/crypto/asn1/tasn_utl.c',
    'third_party/boringssl/crypto/asn1/time_support.c',
    'third_party/boringssl/crypto/base64/base64.c',
    'third_party/boringssl/crypto/bio/bio.c',
    'third_party/boringssl/crypto/bio/bio_mem.c',
    'third_party/boringssl/crypto/bio/connect.c',
    'third_party/boringssl/crypto/bio/fd.c',
    'third_party/boringssl/crypto/bio/file.c',
    'third_party/boringssl/crypto/bio/hexdump.c',
    'third_party/boringssl/crypto/bio/pair.c',
    'third_party/boringssl/crypto/bio/printf.c',
    'third_party/boringssl/crypto/bio/socket.c',
    'third_party/boringssl/crypto/bio/socket_helper.c',
    'third_party/boringssl/crypto/bn_extra/bn_asn1.c',
    'third_party/boringssl/crypto/bn_extra/convert.c',
    'third_party/boringssl/crypto/buf/buf.c',
    'third_party/boringssl/crypto/bytestring/asn1_compat.c',
    'third_party/boringssl/crypto/bytestring/ber.c',
    'third_party/boringssl/crypto/bytestring/cbb.c',
    'third_party/boringssl/crypto/bytestring/cbs.c',
    'third_party/boringssl/crypto/chacha/chacha.c',
    'third_party/boringssl/crypto/cipher_extra/cipher_extra.c',
    'third_party/boringssl/crypto/cipher_extra/derive_key.c',
    'third_party/boringssl/crypto/cipher_extra/e_aesccm.c',
    'third_party/boringssl/crypto/cipher_extra/e_aesctrhmac.c',
    'third_party/boringssl/crypto/cipher_extra/e_aesgcmsiv.c',
    'third_party/boringssl/crypto/cipher_extra/e_chacha20poly1305.c',
    'third_party/boringssl/crypto/cipher_extra/e_null.c',
    'third_party/boringssl/crypto/cipher_extra/e_rc2.c',
    'third_party/boringssl/crypto/cipher_extra/e_rc4.c',
    'third_party/boringssl/crypto/cipher_extra/e_ssl3.c',
    'third_party/boringssl/crypto/cipher_extra/e_tls.c',
    'third_party/boringssl/crypto/cipher_extra/tls_cbc.c',
    'third_party/boringssl/crypto/cmac/cmac.c',
    'third_party/boringssl/crypto/conf/conf.c',
    'third_party/boringssl/crypto/cpu-aarch64-fuchsia.c',
    'third_party/boringssl/crypto/cpu-aarch64-linux.c',
    'third_party/boringssl/crypto/cpu-arm-linux.c',
    'third_party/boringssl/crypto/cpu-arm.c',
    'third_party/boringssl/crypto/cpu-intel.c',
    'third_party/boringssl/crypto/cpu-ppc64le.c',
    'third_party/boringssl/crypto/crypto.c',
    'third_party/boringssl/crypto/curve25519/spake25519.c',
    'third_party/boringssl/crypto/dh/check.c',
    'third_party/boringssl/crypto/dh/dh.c',
    'third_party/boringssl/crypto/dh/dh_asn1.c',
    'third_party/boringssl/crypto/dh/params.c',
    'third_party/boringssl/crypto/digest_extra/digest_extra.c',
    'third_party/boringssl/crypto/dsa/dsa.c',
    'third_party/boringssl/crypto/dsa/dsa_asn1.c',
    'third_party/boringssl/crypto/ec_extra/ec_asn1.c',
    'third_party/boringssl/crypto/ecdh/ecdh.c',
    'third_party/boringssl/crypto/ecdsa_extra/ecdsa_asn1.c',
    'third_party/boringssl/crypto/engine/engine.c',
    'third_party/boringssl/crypto/err/err.c',
    'third_party/boringssl/crypto/evp/digestsign.c',
    'third_party/boringssl/crypto/evp/evp.c',
    'third_party/boringssl/crypto/evp/evp_asn1.c',
    'third_party/boringssl/crypto/evp/evp_ctx.c',
    'third_party/boringssl/crypto/evp/p_dsa_asn1.c',
    'third_party/boringssl/crypto/evp/p_ec.c',
    'third_party/boringssl/crypto/evp/p_ec_asn1.c',
    'third_party/boringssl/crypto/evp/p_ed25519.c',
    'third_party/boringssl/crypto/evp/p_ed25519_asn1.c',
    'third_party/boringssl/crypto/evp/p_rsa.c',
    'third_party/boringssl/crypto/evp/p_rsa_asn1.c',
    'third_party/boringssl/crypto/evp/pbkdf.c',
    'third_party/boringssl/crypto/evp/print.c',
    'third_party/boringssl/crypto/evp/scrypt.c',
    'third_party/boringssl/crypto/evp/sign.c',
    'third_party/boringssl/crypto/ex_data.c',
    'third_party/boringssl/crypto/fipsmodule/bcm.c',
    'third_party/boringssl/crypto/fipsmodule/is_fips.c',
    'third_party/boringssl/crypto/hkdf/hkdf.c',
    'third_party/boringssl/crypto/lhash/lhash.c',
    'third_party/boringssl/crypto/mem.c',
    'third_party/boringssl/crypto/obj/obj.c',
    'third_party/boringssl/crypto/obj/obj_xref.c',
    'third_party/boringssl/crypto/pem/pem_all.c',
    'third_party/boringssl/crypto/pem/pem_info.c',
    'third_party/boringssl/crypto/pem/pem_lib.c',
    'third_party/boringssl/crypto/pem/pem_oth.c',
    'third_party/boringssl/crypto/pem/pem_pk8.c',
    'third_party/boringssl/crypto/pem/pem_pkey.c',
    'third_party/boringssl/crypto/pem/pem_x509.c',
    'third_party/boringssl/crypto/pem/pem_xaux.c',
    'third_party/boringssl/crypto/pkcs7/pkcs7.c',
    'third_party/boringssl/crypto/pkcs7/pkcs7_x509.c',
    'third_party/boringssl/crypto/pkcs8/p5_pbev2.c',
    'third_party/boringssl/crypto/pkcs8/pkcs8.c',
    'third_party/boringssl/crypto/pkcs8/pkcs8_x509.c',
    'third_party/boringssl/crypto/poly1305/poly1305.c',
    'third_party/boringssl/crypto/poly1305/poly1305_arm.c',
    'third_party/boringssl/crypto/poly1305/poly1305_vec.c',
    'third_party/boringssl/crypto/pool/pool.c',
    'third_party/boringssl/crypto/rand_extra/deterministic.c',
    'third_party/boringssl/crypto/rand_extra/forkunsafe.c',
    'third_party/boringssl/crypto/rand_extra/fuchsia.c',
    'third_party/boringssl/crypto/rand_extra/rand_extra.c',
    'third_party/boringssl/crypto/rand_extra/windows.c',
    'third_party/boringssl/crypto/rc4/rc4.c',
    'third_party/boringssl/crypto/refcount_c11.c',
    'third_party/boringssl/crypto/refcount_lock.c',
    'third_party/boringssl/crypto/rsa_extra/rsa_asn1.c',
    'third_party/boringssl/crypto/stack/stack.c',
    'third_party/boringssl/crypto/thread.c',
    'third_party/boringssl/crypto/thread_none.c',
    'third_party/boringssl/crypto/thread_pthread.c',
    'third_party/boringssl/crypto/thread_win.c',
    'third_party/boringssl/crypto/x509/a_digest.c',
    'third_party/boringssl/crypto/x509/a_sign.c',
    'third_party/boringssl/crypto/x509/a_strex.c',
    'third_party/boringssl/crypto/x509/a_verify.c',
    'third_party/boringssl/crypto/x509/algorithm.c',
    'third_party/boringssl/crypto/x509/asn1_gen.c',
    'third_party/boringssl/crypto/x509/by_dir.c',
    'third_party/boringssl/crypto/x509/by_file.c',
    'third_party/boringssl/crypto/x509/i2d_pr.c',
    'third_party/boringssl/crypto/x509/rsa_pss.c',
    'third_party/boringssl/crypto/x509/t_crl.c',
    'third_party/boringssl/crypto/x509/t_req.c',
    'third_party/boringssl/crypto/x509/t_x509.c',
    'third_party/boringssl/crypto/x509/t_x509a.c',
    'third_party/boringssl/crypto/x509/x509.c',
    'third_party/boringssl/crypto/x509/x509_att.c',
    'third_party/boringssl/crypto/x509/x509_cmp.c',
    'third_party/boringssl/crypto/x509/x509_d2.c',
    'third_party/boringssl/crypto/x509/x509_def.c',
    'third_party/boringssl/crypto/x509/x509_ext.c',
    'third_party/boringssl/crypto/x509/x509_lu.c',
    'third_party/boringssl/crypto/x509/x509_obj.c',
    'third_party/boringssl/crypto/x509/x509_r2x.c',
    'third_party/boringssl/crypto/x509/x509_req.c',
    'third_party/boringssl/crypto/x509/x509_set.c',
    'third_party/boringssl/crypto/x509/x509_trs.c',
    'third_party/boringssl/crypto/x509/x509_txt.c',
    'third_party/boringssl/crypto/x509/x509_v3.c',
    'third_party/boringssl/crypto/x509/x509_vfy.c',
    'third_party/boringssl/crypto/x509/x509_vpm.c',
    'third_party/boringssl/crypto/x509/x509cset.c',
    'third_party/boringssl/crypto/x509/x509name.c',
    'third_party/boringssl/crypto/x509/x509rset.c',
    'third_party/boringssl/crypto/x509/x509spki.c',
    'third_party/boringssl/crypto/x509/x_algor.c',
    'third_party/boringssl/crypto/x509/x_all.c',
    'third_party/boringssl/crypto/x509/x_attrib.c',
    'third_party/boringssl/crypto/x509/x_crl.c',
    'third_party/boringssl/crypto/x509/x_exten.c',
    'third_party/boringssl/crypto/x509/x_info.c',
    'third_party/boringssl/crypto/x509/x_name.c',
    'third_party/boringssl/crypto/x509/x_pkey.c',
    'third_party/boringssl/crypto/x509/x_pubkey.c',
    'third_party/boringssl/crypto/x509/x_req.c',
    'third_party/boringssl/crypto/x509/x_sig.c',
    'third_party/boringssl/crypto/x509/x_spki.c',
    'third_party/boringssl/crypto/x509/x_val.c',
    'third_party/boringssl/crypto/x509/x_x509.c',
    'third_party/boringssl/crypto/x509/x_x509a.c',
    'third_party/boringssl/crypto/x509v3/pcy_cache.c',
    'third_party/boringssl/crypto/x509v3/pcy_data.c',
    'third_party/boringssl/crypto/x509v3/pcy_lib.c',
    'third_party/boringssl/crypto/x509v3/pcy_map.c',
    'third_party/boringssl/crypto/x509v3/pcy_node.c',
    'third_party/boringssl/crypto/x509v3/pcy_tree.c',
    'third_party/boringssl/crypto/x509v3/v3_akey.c',
    'third_party/boringssl/crypto/x509v3/v3_akeya.c',
    'third_party/boringssl/crypto/x509v3/v3_alt.c',
    'third_party/boringssl/crypto/x509v3/v3_bcons.c',
    'third_party/boringssl/crypto/x509v3/v3_bitst.c',
    'third_party/boringssl/crypto/x509v3/v3_conf.c',
    'third_party/boringssl/crypto/x509v3/v3_cpols.c',
    'third_party/boringssl/crypto/x509v3/v3_crld.c',
    'third_party/boringssl/crypto/x509v3/v3_enum.c',
    'third_party/boringssl/crypto/x509v3/v3_extku.c',
    'third_party/boringssl/crypto/x509v3/v3_genn.c',
    'third_party/boringssl/crypto/x509v3/v3_ia5.c',
    'third_party/boringssl/crypto/x509v3/v3_info.c',
    'third_party/boringssl/crypto/x509v3/v3_int.c',
    'third_party/boringssl/crypto/x509v3/v3_lib.c',
    'third_party/boringssl/crypto/x509v3/v3_ncons.c',
    'third_party/boringssl/crypto/x509v3/v3_pci.c',
    'third_party/boringssl/crypto/x509v3/v3_pcia.c',
    'third_party/boringssl/crypto/x509v3/v3_pcons.c',
    'third_party/boringssl/crypto/x509v3/v3_pku.c',
    'third_party/boringssl/crypto/x509v3/v3_pmaps.c',
    'third_party/boringssl/crypto/x509v3/v3_prn.c',
    'third_party/boringssl/crypto/x509v3/v3_purp.c',
    'third_party/boringssl/crypto/x509v3/v3_skey.c',
    'third_party/boringssl/crypto/x509v3/v3_sxnet.c',
    'third_party/boringssl/crypto/x509v3/v3_utl.c',
    'third_party/boringssl/ssl/bio_ssl.cc',
    'third_party/boringssl/ssl/custom_extensions.cc',
    'third_party/boringssl/ssl/d1_both.cc',
    'third_party/boringssl/ssl/d1_lib.cc',
    'third_party/boringssl/ssl/d1_pkt.cc',
    'third_party/boringssl/ssl/d1_srtp.cc',
    'third_party/boringssl/ssl/dtls_method.cc',
    'third_party/boringssl/ssl/dtls_record.cc',
    'third_party/boringssl/ssl/handoff.cc',
    'third_party/boringssl/ssl/handshake.cc',
    'third_party/boringssl/ssl/handshake_client.cc',
    'third_party/boringssl/ssl/handshake_server.cc',
    'third_party/boringssl/ssl/s3_both.cc',
    'third_party/boringssl/ssl/s3_lib.cc',
    'third_party/boringssl/ssl/s3_pkt.cc',
    'third_party/boringssl/ssl/ssl_aead_ctx.cc',
    'third_party/boringssl/ssl/ssl_asn1.cc',
    'third_party/boringssl/ssl/ssl_buffer.cc',
    'third_party/boringssl/ssl/ssl_cert.cc',
    'third_party/boringssl/ssl/ssl_cipher.cc',
    'third_party/boringssl/ssl/ssl_file.cc',
    'third_party/boringssl/ssl/ssl_key_share.cc',
    'third_party/boringssl/ssl/ssl_lib.cc',
    'third_party/boringssl/ssl/ssl_privkey.cc',
    'third_party/boringssl/ssl/ssl_session.cc',
    'third_party/boringssl/ssl/ssl_stat.cc',
    'third_party/boringssl/ssl/ssl_transcript.cc',
    'third_party/boringssl/ssl/ssl_versions.cc',
    'third_party/boringssl/ssl/ssl_x509.cc',
    'third_party/boringssl/ssl/t1_enc.cc',
    'third_party/boringssl/ssl/t1_lib.cc',
    'third_party/boringssl/ssl/tls13_both.cc',
    'third_party/boringssl/ssl/tls13_client.cc',
    'third_party/boringssl/ssl/tls13_enc.cc',
    'third_party/boringssl/ssl/tls13_server.cc',
    'third_party/boringssl/ssl/tls_method.cc',
    'third_party/boringssl/ssl/tls_record.cc',
    'third_party/boringssl/third_party/fiat/curve25519.c',
    'third_party/zlib/adler32.c',
    'third_party/zlib/compress.c',
    'third_party/zlib/crc32.c',
    'third_party/zlib/deflate.c',
    'third_party/zlib/gzclose.c',
    'third_party/zlib/gzlib.c',
    'third_party/zlib/gzread.c',
    'third_party/zlib/gzwrite.c',
    'third_party/zlib/infback.c',
    'third_party/zlib/inffast.c',
    'third_party/zlib/inflate.c',
    'third_party/zlib/inftrees.c',
    'third_party/zlib/trees.c',
    'third_party/zlib/uncompr.c',
    'third_party/zlib/zutil.c',
    'third_party/cares/cares/ares__close_sockets.c',
    'third_party/cares/cares/ares__get_hostent.c',
    'third_party/cares/cares/ares__read_line.c',
    'third_party/cares/cares/ares__timeval.c',
    'third_party/cares/cares/ares_cancel.c',
    'third_party/cares/cares/ares_create_query.c',
    'third_party/cares/cares/ares_data.c',
    'third_party/cares/cares/ares_destroy.c',
    'third_party/cares/cares/ares_expand_name.c',
    'third_party/cares/cares/ares_expand_string.c',
    'third_party/cares/cares/ares_fds.c',
    'third_party/cares/cares/ares_free_hostent.c',
    'third_party/cares/cares/ares_free_string.c',
    'third_party/cares/cares/ares_getenv.c',
    'third_party/cares/cares/ares_gethostbyaddr.c',
    'third_party/cares/cares/ares_gethostbyname.c',
    'third_party/cares/cares/ares_getnameinfo.c',
    'third_party/cares/cares/ares_getopt.c',
    'third_party/cares/cares/ares_getsock.c',
    'third_party/cares/cares/ares_init.c',
    'third_party/cares/cares/ares_library_init.c',
    'third_party/cares/cares/ares_llist.c',
    'third_party/cares/cares/ares_mkquery.c',
    'third_party/cares/cares/ares_nowarn.c',
    'third_party/cares/cares/ares_options.c',
    'third_party/cares/cares/ares_parse_a_reply.c',
    'third_party/cares/cares/ares_parse_aaaa_reply.c',
    'third_party/cares/cares/ares_parse_mx_reply.c',
    'third_party/cares/cares/ares_parse_naptr_reply.c',
    'third_party/cares/cares/ares_parse_ns_reply.c',
    'third_party/cares/cares/ares_parse_ptr_reply.c',
    'third_party/cares/cares/ares_parse_soa_reply.c',
    'third_party/cares/cares/ares_parse_srv_reply.c',
    'third_party/cares/cares/ares_parse_txt_reply.c',
    'third_party/cares/cares/ares_platform.c',
    'third_party/cares/cares/ares_process.c',
    'third_party/cares/cares/ares_query.c',
    'third_party/cares/cares/ares_search.c',
    'third_party/cares/cares/ares_send.c',
    'third_party/cares/cares/ares_strcasecmp.c',
    'third_party/cares/cares/ares_strdup.c',
    'third_party/cares/cares/ares_strerror.c',
    'third_party/cares/cares/ares_strsplit.c',
    'third_party/cares/cares/ares_timeout.c',
    'third_party/cares/cares/ares_version.c',
    'third_party/cares/cares/ares_writev.c',
    'third_party/cares/cares/bitncmp.c',
    'third_party/cares/cares/inet_net_pton.c',
    'third_party/cares/cares/inet_ntop.c',
    'third_party/cares/cares/windows_port.c',
]



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A setup module for the GRPC Python package."""
from distutils import cygwinccompiler
from distutils import extension as _extension
from distutils import util
import os
import os.path
import pkg_resources
import platform
import re
import shlex
import shutil
import sys
import sysconfig

import setuptools
from setuptools.command import egg_info

# Redirect the manifest template from MANIFEST.in to PYTHON-MANIFEST.in.
egg_info.manifest_maker.template = 'PYTHON-MANIFEST.in'

PY3 = sys.version_info.major == 3
PYTHON_STEM = os.path.join('src', 'python', 'grpcio')
CORE_INCLUDE = ('include', '.',)
SSL_INCLUDE = (os.path.join('third_party', 'boringssl', 'include'),)
ZLIB_INCLUDE = (os.path.join('third_party', 'zlib'),)
NANOPB_INCLUDE = (os.path.join('third_party', 'nanopb'),)
CARES_INCLUDE = (
    os.path.join('third_party', 'cares'),
    os.path.join('third_party', 'cares', 'cares'),)
if 'darwin' in sys.platform:
  CARES_INCLUDE += (os.path.join('third_party', 'cares', 'config_darwin'),)
if 'freebsd' in sys.platform:
  CARES_INCLUDE += (os.path.join('third_party', 'cares', 'config_freebsd'),)
if 'linux' in sys.platform:
  CARES_INCLUDE += (os.path.join('third_party', 'cares', 'config_linux'),)
if 'openbsd' in sys.platform:
  CARES_INCLUDE += (os.path.join('third_party', 'cares', 'config_openbsd'),)
ADDRESS_SORTING_INCLUDE = (os.path.join('third_party', 'address_sorting', 'include'),)
README = os.path.join(PYTHON_STEM, 'README.rst')

# Ensure we're in the proper directory whether or not we're being used by pip.
os.chdir(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, os.path.abspath(PYTHON_STEM))

# Break import-style to ensure we can actually find our in-repo dependencies.
import _parallel_compile_patch
import _spawn_patch
import commands
import grpc_core_dependencies
import grpc_version

_parallel_compile_patch.monkeypatch_compile_maybe()
_spawn_patch.monkeypatch_spawn()

LICENSE = 'Apache License 2.0'

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: Apache Software License',
]

# Environment variable to determine whether or not the Cython extension should
# *use* Cython or use the generated C files. Note that this requires the C files
# to have been generated by building first *with* Cython support. Even if this
# is set to false, if the script detects that the generated `.c` file isn't
# present, then it will still attempt to use Cython.
BUILD_WITH_CYTHON = os.environ.get('GRPC_PYTHON_BUILD_WITH_CYTHON', False)


# Export this variable to use the system installation of openssl. You need to
# have the header files installed (in /usr/include/openssl) and during
# runtime, the shared library must be installed
BUILD_WITH_SYSTEM_OPENSSL = os.environ.get('GRPC_PYTHON_BUILD_SYSTEM_OPENSSL',
                                           False)

# Export this variable to use the system installation of zlib. You need to
# have the header files installed (in /usr/include/) and during
# runtime, the shared library must be installed
BUILD_WITH_SYSTEM_ZLIB = os.environ.get('GRPC_PYTHON_BUILD_SYSTEM_ZLIB',
                                        False)

# Export this variable to use the system installation of cares. You need to
# have the header files installed (in /usr/include/) and during
# runtime, the shared library must be installed
BUILD_WITH_SYSTEM_CARES = os.environ.get('GRPC_PYTHON_BUILD_SYSTEM_CARES',
                                         False)

# For local development use only: This skips building gRPC Core and its
# dependencies, including protobuf and boringssl. This allows "incremental"
# compilation by first building gRPC Core using make, then building only the
# Python/Cython layers here.
#
# Note that this requires libboringssl.a in the libs/{dbg,opt}/ directory, which
# may require configuring make to not use the system openssl implementation:
#
#    make HAS_SYSTEM_OPENSSL_ALPN=0
#
# TODO(ericgribkoff) Respect the BUILD_WITH_SYSTEM_* flags alongside this option
USE_PREBUILT_GRPC_CORE = os.environ.get(
    'GRPC_PYTHON_USE_PREBUILT_GRPC_CORE', False)


# If this environmental variable is set, GRPC will not try to be compatible with
# libc versions old than the one it was compiled against.
DISABLE_LIBC_COMPATIBILITY = os.environ.get('GRPC_PYTHON_DISABLE_LIBC_COMPATIBILITY', False)

# Environment variable to determine whether or not to enable coverage analysis
# in Cython modules.
ENABLE_CYTHON_TRACING = os.environ.get(
    'GRPC_PYTHON_ENABLE_CYTHON_TRACING', False)

# Environment variable specifying whether or not there's interest in setting up
# documentation building.
ENABLE_DOCUMENTATION_BUILD = os.environ.get(
    'GRPC_PYTHON_ENABLE_DOCUMENTATION_BUILD', False)

# There are some situations (like on Windows) where CC, CFLAGS, and LDFLAGS are
# entirely ignored/dropped/forgotten by distutils and its Cygwin/MinGW support.
# We use these environment variables to thus get around that without locking
# ourselves in w.r.t. the multitude of operating systems this ought to build on.
# We can also use these variables as a way to inject environment-specific
# compiler/linker flags. We assume GCC-like compilers and/or MinGW as a
# reasonable default.
EXTRA_ENV_COMPILE_ARGS = os.environ.get('GRPC_PYTHON_CFLAGS', None)
EXTRA_ENV_LINK_ARGS = os.environ.get('GRPC_PYTHON_LDFLAGS', None)
if EXTRA_ENV_COMPILE_ARGS is None:
  EXTRA_ENV_COMPILE_ARGS = ' -std=c++11'
  if 'win32' in sys.platform and sys.version_info < (3, 5):
    EXTRA_ENV_COMPILE_ARGS += ' -D_hypot=hypot'
    # We use define flags here and don't directly add to DEFINE_MACROS below to
    # ensure that the expert user/builder has a way of turning it off (via the
    # envvars) without adding yet more GRPC-specific envvars.
    # See https://sourceforge.net/p/mingw-w64/bugs/363/
    if '32' in platform.architecture()[0]:
      EXTRA_ENV_COMPILE_ARGS += ' -D_ftime=_ftime32 -D_timeb=__timeb32 -D_ftime_s=_ftime32_s'
    else:
      EXTRA_ENV_COMPILE_ARGS += ' -D_ftime=_ftime64 -D_timeb=__timeb64'
  elif "linux" in sys.platform:
    EXTRA_ENV_COMPILE_ARGS += ' -std=gnu99 -fvisibility=hidden -fno-wrapv -fno-exceptions'
  elif "darwin" in sys.platform:
    EXTRA_ENV_COMPILE_ARGS += ' -stdlib=libc++ -fvisibility=hidden -fno-wrapv -fno-exceptions'
EXTRA_ENV_COMPILE_ARGS += ' -DPB_FIELD_32BIT'

if EXTRA_ENV_LINK_ARGS is None:
  EXTRA_ENV_LINK_ARGS = ''
  if "linux" in sys.platform or "darwin" in sys.platform:
    EXTRA_ENV_LINK_ARGS += ' -lpthread'
  elif "win32" in sys.platform and sys.version_info < (3, 5):
    msvcr = cygwinccompiler.get_msvcr()[0]
    # TODO(atash) sift through the GCC specs to see if libstdc++ can have any
    # influence on the linkage outcome on MinGW for non-C++ programs.
    EXTRA_ENV_LINK_ARGS += (
        ' -static-libgcc -static-libstdc++ -mcrtdll={msvcr} '
        '-static'.format(msvcr=msvcr))
  if "linux" in sys.platform:
    EXTRA_ENV_LINK_ARGS += ' -Wl,-wrap,memcpy  -static-libgcc'

EXTRA_COMPILE_ARGS = shlex.split(EXTRA_ENV_COMPILE_ARGS)
EXTRA_LINK_ARGS = shlex.split(EXTRA_ENV_LINK_ARGS)

CYTHON_EXTENSION_PACKAGE_NAMES = ()

CYTHON_EXTENSION_MODULE_NAMES = ('grpc._cython.cygrpc',)

CYTHON_HELPER_C_FILES = ()

CORE_C_FILES = tuple(grpc_core_dependencies.CORE_SOURCE_FILES)
if "win32" in sys.platform:
  CORE_C_FILES = filter(lambda x: 'third_party/cares' not in x, CORE_C_FILES)

if BUILD_WITH_SYSTEM_OPENSSL:
  CORE_C_FILES = filter(lambda x: 'third_party/boringssl' not in x, CORE_C_FILES)
  CORE_C_FILES = filter(lambda x: 'src/boringssl' not in x, CORE_C_FILES)
  SSL_INCLUDE = (os.path.join('/usr', 'include', 'openssl'),)

if BUILD_WITH_SYSTEM_ZLIB:
  CORE_C_FILES = filter(lambda x: 'third_party/zlib' not in x, CORE_C_FILES)
  ZLIB_INCLUDE = (os.path.join('/usr', 'include'),)

if BUILD_WITH_SYSTEM_CARES:
  CORE_C_FILES = filter(lambda x: 'third_party/cares' not in x, CORE_C_FILES)
  CARES_INCLUDE = (os.path.join('/usr', 'include'),)

EXTENSION_INCLUDE_DIRECTORIES = (
    (PYTHON_STEM,) + CORE_INCLUDE + SSL_INCLUDE + ZLIB_INCLUDE +
    NANOPB_INCLUDE + CARES_INCLUDE + ADDRESS_SORTING_INCLUDE)

EXTENSION_LIBRARIES = ()
if "linux" in sys.platform:
  EXTENSION_LIBRARIES += ('rt',)
if not "win32" in sys.platform:
  EXTENSION_LIBRARIES += ('m',)
if "win32" in sys.platform:
  EXTENSION_LIBRARIES += ('advapi32', 'ws2_32',)
if BUILD_WITH_SYSTEM_OPENSSL:
  EXTENSION_LIBRARIES += ('ssl', 'crypto',)
if BUILD_WITH_SYSTEM_ZLIB:
  EXTENSION_LIBRARIES += ('z',)
if BUILD_WITH_SYSTEM_CARES:
  EXTENSION_LIBRARIES += ('cares',)

DEFINE_MACROS = (('OPENSSL_NO_ASM', 1), ('_WIN32_WINNT', 0x600))
if not DISABLE_LIBC_COMPATIBILITY:
  DEFINE_MACROS += (('GPR_BACKWARDS_COMPATIBILITY_MODE', 1),)
if "win32" in sys.platform:
  # TODO(zyc): Re-enable c-ares on x64 and x86 windows after fixing the
  # ares_library_init compilation issue
  DEFINE_MACROS += (('WIN32_LEAN_AND_MEAN', 1), ('CARES_STATICLIB', 1),
                    ('GRPC_ARES', 0), ('NTDDI_VERSION', 0x06000000),
                    ('NOMINMAX', 1),)
  if '64bit' in platform.architecture()[0]:
    DEFINE_MACROS += (('MS_WIN64', 1),)
  elif sys.version_info >= (3, 5):
    # For some reason, this is needed to get access to inet_pton/inet_ntop
    # on msvc, but only for 32 bits
    DEFINE_MACROS += (('NTDDI_VERSION', 0x06000000),)
else:
  DEFINE_MACROS += (('HAVE_CONFIG_H', 1), ('GRPC_ENABLE_FORK_SUPPORT', 1),)

LDFLAGS = tuple(EXTRA_LINK_ARGS)
CFLAGS = tuple(EXTRA_COMPILE_ARGS)
if "linux" in sys.platform or "darwin" in sys.platform:
  pymodinit_type = 'PyObject*' if PY3 else 'void'
  pymodinit = 'extern "C" __attribute__((visibility ("default"))) {}'.format(pymodinit_type)
  DEFINE_MACROS += (('PyMODINIT_FUNC', pymodinit),)
  DEFINE_MACROS += (('GRPC_POSIX_FORK_ALLOW_PTHREAD_ATFORK', 1),)

# By default, Python3 distutils enforces compatibility of
# c plugins (.so files) with the OSX version Python3 was built with.
# For Python3.4, this is OSX 10.6, but we need Thread Local Support (__thread)
if 'darwin' in sys.platform and PY3:
  mac_target = sysconfig.get_config_var('MACOSX_DEPLOYMENT_TARGET')
  if mac_target and (pkg_resources.parse_version(mac_target) <
                     pkg_resources.parse_version('10.7.0')):
    os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.7'
    os.environ['_PYTHON_HOST_PLATFORM'] = re.sub(
        r'macosx-[0-9]+\.[0-9]+-(.+)',
        r'macosx-10.7-\1',
        util.get_platform())

def cython_extensions_and_necessity():
  cython_module_files = [os.path.join(PYTHON_STEM,
                               name.replace('.', '/') + '.pyx')
                  for name in CYTHON_EXTENSION_MODULE_NAMES]
  config = os.environ.get('CONFIG', 'opt')
  prefix = 'libs/' + config + '/'
  if USE_PREBUILT_GRPC_CORE:
    extra_objects = [prefix + 'libares.a',
                     prefix + 'libboringssl.a',
                     prefix + 'libgpr.a',
                     prefix + 'libgrpc.a']
    core_c_files = []
  else:
    core_c_files = list(CORE_C_FILES)
    extra_objects = []
  extensions = [
      _extension.Extension(
          name=module_name,
          sources=[module_file] + list(CYTHON_HELPER_C_FILES) + core_c_files,
          include_dirs=list(EXTENSION_INCLUDE_DIRECTORIES),
          libraries=list(EXTENSION_LIBRARIES),
          define_macros=list(DEFINE_MACROS),
          extra_objects=extra_objects,
          extra_compile_args=list(CFLAGS),
          extra_link_args=list(LDFLAGS),
      ) for (module_name, module_file) in zip(list(CYTHON_EXTENSION_MODULE_NAMES), cython_module_files)
  ]
  need_cython = BUILD_WITH_CYTHON
  if not BUILD_WITH_CYTHON:
    need_cython = need_cython or not commands.check_and_update_cythonization(extensions)
  return commands.try_cythonize(extensions, linetracing=ENABLE_CYTHON_TRACING, mandatory=BUILD_WITH_CYTHON), need_cython

CYTHON_EXTENSION_MODULES, need_cython = cython_extensions_and_necessity()

PACKAGE_DIRECTORIES = {
    '': PYTHON_STEM,
}

INSTALL_REQUIRES = (
    "six>=1.5.2",
    "futures>=2.2.0; python_version<'3.2'",
    "enum34>=1.0.4; python_version<'3.4'",
)

SETUP_REQUIRES = INSTALL_REQUIRES + (
    'Sphinx~=1.8.1',
    'six>=1.10',
  ) if ENABLE_DOCUMENTATION_BUILD else ()

try:
  import Cython
except ImportError:
  if BUILD_WITH_CYTHON:
    sys.stderr.write(
      "You requested a Cython build via GRPC_PYTHON_BUILD_WITH_CYTHON, "
      "but do not have Cython installed. We won't stop you from using "
      "other commands, but the extension files will fail to build.\n")
  elif need_cython:
    sys.stderr.write(
        'We could not find Cython. Setup may take 10-20 minutes.\n')
    SETUP_REQUIRES += ('cython>=0.23',)

COMMAND_CLASS = {
    'doc': commands.SphinxDocumentation,
    'build_project_metadata': commands.BuildProjectMetadata,
    'build_py': commands.BuildPy,
    'build_ext': commands.BuildExt,
    'gather': commands.Gather,
}

# Ensure that package data is copied over before any commands have been run:
credentials_dir = os.path.join(PYTHON_STEM, 'grpc', '_cython', '_credentials')
try:
  os.mkdir(credentials_dir)
except OSError:
  pass
shutil.copyfile(os.path.join('etc', 'roots.pem'),
                os.path.join(credentials_dir, 'roots.pem'))

PACKAGE_DATA = {
    # Binaries that may or may not be present in the final installation, but are
    # mentioned here for completeness.
    'grpc._cython': [
        '_credentials/roots.pem',
        '_windows/grpc_c.32.python',
        '_windows/grpc_c.64.python',
    ],
}
PACKAGES = setuptools.find_packages(PYTHON_STEM)

setuptools.setup(
  name='grpcio',
  version=grpc_version.VERSION,
  description='HTTP/2-based RPC framework',
  author='The gRPC Authors',
  author_email='grpc-io@googlegroups.com',
  url='https://grpc.io',
  license=LICENSE,
  classifiers=CLASSIFIERS,
  long_description=open(README).read(),
  ext_modules=CYTHON_EXTENSION_MODULES,
  packages=list(PACKAGES),
  package_dir=PACKAGE_DIRECTORIES,
  package_data=PACKAGE_DATA,
  install_requires=INSTALL_REQUIRES,
  setup_requires=SETUP_REQUIRES,
  cmdclass=COMMAND_CLASS,
)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef grpc_event _next(grpc_completion_queue *c_completion_queue, deadline) except *


cdef _interpret_event(grpc_event c_event)


cdef class CompletionQueue:

  cdef grpc_completion_queue *c_completion_queue
  cdef bint is_shutting_down
  cdef bint is_shutdown

  cdef _interpret_event(self, grpc_event c_event)



'''
Scons Builder for nanopb .proto definitions.

This tool will locate the nanopb generator and use it to generate .pb.c and
.pb.h files from the .proto files.

Basic example
-------------
# Build myproto.pb.c and myproto.pb.h from myproto.proto
myproto = env.NanopbProto("myproto")

# Link nanopb core to the program
env.Append(CPPPATH = "$NANOB")
myprog = env.Program(["myprog.c", myproto, "$NANOPB/pb_encode.c", "$NANOPB/pb_decode.c"])

Configuration options
---------------------
Normally, this script is used in the test environment of nanopb and it locates
the nanopb generator by a relative path. If this script is used in another
application, the path to nanopb root directory has to be defined:

env.SetDefault(NANOPB = "path/to/nanopb")

Additionally, the path to protoc and the options to give to protoc can be
defined manually:

env.SetDefault(PROTOC = "path/to/protoc")
env.SetDefault(PROTOCFLAGS = "--plugin=protoc-gen-nanopb=path/to/protoc-gen-nanopb")
'''

import SCons.Action
import SCons.Builder
import SCons.Util
import os.path

class NanopbWarning(SCons.Warnings.Warning):
    pass
SCons.Warnings.enableWarningClass(NanopbWarning)

def _detect_nanopb(env):
    '''Find the path to nanopb root directory.'''
    if env.has_key('NANOPB'):
        # Use nanopb dir given by user
        return env['NANOPB']
    
    p = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..'))
    if os.path.isdir(p) and os.path.isfile(os.path.join(p, 'pb.h')):
        # Assume we are running under tests/site_scons/site_tools
        return p
    
    raise SCons.Errors.StopError(NanopbWarning,
        "Could not find the nanopb root directory")

def _detect_protoc(env):
    '''Find the path to the protoc compiler.'''
    if env.has_key('PROTOC'):
        # Use protoc defined by user
        return env['PROTOC']
    
    n = _detect_nanopb(env)
    p1 = os.path.join(n, 'generator-bin', 'protoc' + env['PROGSUFFIX'])
    if os.path.exists(p1):
        # Use protoc bundled with binary package
        return env['ESCAPE'](p1)
    
    p = env.WhereIs('protoc')
    if p:
        # Use protoc from path
        return env['ESCAPE'](p)
    
    raise SCons.Errors.StopError(NanopbWarning,
        "Could not find the protoc compiler")

def _detect_protocflags(env):
    '''Find the options to use for protoc.'''
    if env.has_key('PROTOCFLAGS'):
        return env['PROTOCFLAGS']
    
    p = _detect_protoc(env)
    n = _detect_nanopb(env)
    p1 = os.path.join(n, 'generator-bin', 'protoc' + env['PROGSUFFIX'])
    if p == env['ESCAPE'](p1):
        # Using the bundled protoc, no options needed
        return ''
    
    e = env['ESCAPE']
    if env['PLATFORM'] == 'win32':
        return e('--plugin=protoc-gen-nanopb=' + os.path.join(n, 'generator', 'protoc-gen-nanopb.bat'))
    else:
        return e('--plugin=protoc-gen-nanopb=' + os.path.join(n, 'generator', 'protoc-gen-nanopb'))

def _nanopb_proto_actions(source, target, env, for_signature):
    esc = env['ESCAPE']
    dirs = ' '.join(['-I' + esc(env.GetBuildPath(d)) for d in env['PROTOCPATH']])
    return '$PROTOC $PROTOCFLAGS %s --nanopb_out=. %s' % (dirs, esc(str(source[0])))

def _nanopb_proto_emitter(target, source, env):
    basename = os.path.splitext(str(source[0]))[0]
    target.append(basename + '.pb.h')
    
    if os.path.exists(basename + '.options'):
        source.append(basename + '.options')
    
    return target, source

_nanopb_proto_builder = SCons.Builder.Builder(
    generator = _nanopb_proto_actions,
    suffix = '.pb.c',
    src_suffix = '.proto',
    emitter = _nanopb_proto_emitter)
       
def generate(env):
    '''Add Builder for nanopb protos.'''
    
    env['NANOPB'] = _detect_nanopb(env)
    env['PROTOC'] = _detect_protoc(env)
    env['PROTOCFLAGS'] = _detect_protocflags(env)
    
    env.SetDefault(PROTOCPATH = ['.', os.path.join(env['NANOPB'], 'generator', 'proto')])
    
    env.SetDefault(NANOPB_PROTO_CMD = '$PROTOC $PROTOCFLAGS --nanopb_out=. $SOURCES')
    env['BUILDERS']['NanopbProto'] = _nanopb_proto_builder
    
def exists(env):
    return _detect_protoc(env) and _detect_protoc_opts(env)




#!/usr/bin/env python2.7
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import shutil
import sys
import os
import yaml

sys.dont_write_bytecode = True

boring_ssl_root = os.path.abspath(os.path.join(
    os.path.dirname(sys.argv[0]),
    '../../third_party/boringssl'))
sys.path.append(os.path.join(boring_ssl_root, 'util'))

try:
  import generate_build_files
except ImportError:
  print yaml.dump({})
  sys.exit()

def map_dir(filename):
  if filename[0:4] == 'src/':
    return 'third_party/boringssl/' + filename[4:]
  else:
    return 'src/boringssl/' + filename

def map_testarg(arg):
  if '/' in arg:
    return 'third_party/boringssl/' + arg
  else:
    return arg

class Grpc(object):

  yaml = None

  def WriteFiles(self, files, asm_outputs):
    test_binaries = ['ssl_test', 'crypto_test']

    self.yaml = {
      '#': 'generated with tools/buildgen/gen_boring_ssl_build_yaml.py',
      'raw_boringssl_build_output_for_debugging': {
        'files': files,
        'asm_outputs': asm_outputs,
      },
      'libs': [
          {
            'name': 'boringssl',
            'build': 'private',
            'language': 'c',
            'secure': 'no',
            'src': sorted(
              map_dir(f)
              for f in files['ssl'] + files['crypto']
            ),
            'headers': sorted(
              map_dir(f)
              # We want to include files['fips_fragments'], but not build them as objects.
              # See https://boringssl-review.googlesource.com/c/boringssl/+/16946
              for f in files['ssl_headers'] + files['ssl_internal_headers'] + files['crypto_headers'] + files['crypto_internal_headers'] + files['fips_fragments']
            ),
            'boringssl': True,
            'defaults': 'boringssl',
          },
          {
            'name': 'boringssl_test_util',
            'build': 'private',
            'language': 'c++',
            'secure': 'no',
            'boringssl': True,
            'defaults': 'boringssl',
            'src': [
              map_dir(f)
              for f in sorted(files['test_support'])
            ],
          }
      ],
      'targets': [
          {
            'name': 'boringssl_%s' % test,
            'build': 'test',
            'run': False,
            'secure': 'no',
            'language': 'c++',
            'src': sorted(map_dir(f) for f in files[test]),
            'vs_proj_dir': 'test/boringssl',
            'boringssl': True,
            'defaults': 'boringssl',
            'deps': [
                'boringssl_test_util',
                'boringssl',
            ]
          }
          for test in test_binaries
      ],
      'tests': [
          {
            'name': 'boringssl_%s' % test,
            'args': [],
            'exclude_configs': ['asan', 'ubsan'],
            'ci_platforms': ['linux', 'mac', 'posix', 'windows'],
            'platforms': ['linux', 'mac', 'posix', 'windows'],
            'flaky': False,
            'gtest': True,
            'language': 'c++',
            'boringssl': True,
            'defaults': 'boringssl',
            'cpu_cost': 1.0
          }
          for test in test_binaries
      ]
    }


os.chdir(os.path.dirname(sys.argv[0]))
os.mkdir('src')
try:
  for f in os.listdir(boring_ssl_root):
    os.symlink(os.path.join(boring_ssl_root, f),
               os.path.join('src', f))

  g = Grpc()
  generate_build_files.main([g])

  print yaml.dump(g.yaml)

finally:
  shutil.rmtree('src')
